@@ -3,59 +3,91 @@
  * to mode-specific functions.                                             *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
+ *                                                                         *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
  * continued development of Nmap.  Please email sales@insecure.com for     *
  * further information.                                                    *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
@@ -67,8 +99,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -82,14 +114,13 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_main.c 28195 2012-03-01 09:05:02Z henri $ */
+/* $Id: ncat_main.c 31567 2013-07-29 00:03:01Z david $ */
 
 #include "nsock.h"
 #include "ncat.h"
@@ -115,16 +146,21 @@
 #include <openssl/err.h>
 #endif
 
+#ifdef HAVE_LUA
+#include "ncat_lua.h"
+#endif
+
 static int ncat_connect_mode(void);
 static int ncat_listen_mode(void);
 
 /* Determines if it's parsing HTTP or SOCKS by looking at defport */
-static void parseproxy(char *str, struct sockaddr_storage *ss, unsigned short defport)
+static size_t parseproxy(char *str, struct sockaddr_storage *ss, unsigned short defport)
 {
     char *c = strrchr(str, ':'), *ptr;
     int httpproxy = (defport == DEFAULT_PROXY_PORT);
     unsigned short portno;
     size_t sslen;
+    int rc;
 
     ptr = str;
 
@@ -136,12 +172,15 @@ static void parseproxy(char *str, struct sockaddr_storage *ss, unsigned short de
     else
         portno = defport;
 
-    if (!resolve(ptr, portno, ss, &sslen, o.af)) {
-        loguser("Could not resolve proxy \"%s\".\n", ptr);
+    rc = resolve(ptr, portno, ss, &sslen, o.af);
+    if (rc != 0) {
+        loguser("Could not resolve proxy \"%s\": %s.\n", ptr, gai_strerror(rc));
         if (o.af == AF_INET6 && httpproxy)
             loguser("Did you specify the port number? It's required for IPv6.\n");
         exit(EXIT_FAILURE);
     }
+
+    return sslen;
 }
 
 /* These functions implement a simple linked list to hold allow/deny
@@ -227,11 +266,18 @@ int main(int argc, char *argv[])
     struct option long_options[] = {
         {"4",               no_argument,        NULL,         '4'},
         {"6",               no_argument,        NULL,         '6'},
+#if HAVE_SYS_UN_H
+        {"unixsock",        no_argument,        NULL,         'U'},
+#endif
         {"crlf",            no_argument,        NULL,         'C'},
         {"g",               required_argument,  NULL,         'g'},
         {"G",               required_argument,  NULL,         'G'},
         {"exec",            required_argument,  NULL,         'e'},
         {"sh-exec",         required_argument,  NULL,         'c'},
+#ifdef HAVE_LUA
+        {"lua-exec",        required_argument,  NULL,         0},
+        {"lua-exec-internal",required_argument, NULL,         0},
+#endif
         {"max-conns",       required_argument,  NULL,         'm'},
         {"help",            no_argument,        NULL,         'h'},
         {"delay",           required_argument,  NULL,         'd'},
@@ -254,7 +300,7 @@ int main(int argc, char *argv[])
         {"allowfile",       required_argument,  NULL,         0},
         {"telnet",          no_argument,        NULL,         't'},
         {"udp",             no_argument,        NULL,         'u'},
-        {"sctp",            no_argument,        &o.sctp,      1},
+        {"sctp",            no_argument,        NULL,         0},
         {"version",         no_argument,        NULL,         0},
         {"verbose",         no_argument,        NULL,         'v'},
         {"wait",            required_argument,  NULL,         'w'},
@@ -284,7 +330,7 @@ int main(int argc, char *argv[])
     while (1) {
         /* handle command line arguments */
         int option_index;
-        int c = getopt_long(argc, argv, "46Cc:e:g:G:i:km:hp:d:lo:x:ts:uvw:n",
+        int c = getopt_long(argc, argv, "46UCc:e:g:G:i:km:hp:d:lo:x:ts:uvw:n",
                             long_options, &option_index);
 
         /* That's the end of the options. */
@@ -302,23 +348,38 @@ int main(int argc, char *argv[])
             bye("-6 chosen when IPv6 wasn't compiled in.");
 #endif
             break;
+#if HAVE_SYS_UN_H
+        case 'U':
+            o.af = AF_UNIX;
+            break;
+#endif
         case 'C':
             o.crlf = 1;
             break;
         case 'c':
+            if (o.cmdexec != NULL)
+                bye("Only one of --exec, --sh-exec, and --lua-exec is allowed.");
             o.cmdexec = optarg;
-            o.shellexec = 1;
-	    break;
+            o.execmode = EXEC_SHELL;
+            break;
         case 'e':
+            if (o.cmdexec != NULL)
+                bye("Only one of --exec, --sh-exec, and --lua-exec is allowed.");
             o.cmdexec = optarg;
+            o.execmode = EXEC_PLAIN;
             break;
         case 'g': {
             char *a = strtok(optarg, ",");
             do {
                 union sockaddr_u addr;
                 size_t sslen;
-                if (!resolve(a, 0, &addr.storage, &sslen, AF_INET))
-                    bye("Sorry, could not resolve source route hop %s.", a);
+                int rc;
+
+                rc = resolve(a, 0, &addr.storage, &sslen, AF_INET);
+                if (rc != 0) {
+                    bye("Sorry, could not resolve source route hop \"%s\": %s.",
+                    a, gai_strerror(rc));
+                }
                 o.srcrtes[o.numsrcrtes] = addr.in.sin_addr;
             } while (o.numsrcrtes++ <= 8 && (a = strtok(NULL, ",")));
             if (o.numsrcrtes > 8)
@@ -339,7 +400,7 @@ int main(int argc, char *argv[])
         case 'd':
             o.linedelay = tval2msecs(optarg);
             if (o.linedelay <= 0)
-                bye("Invalid -d delay (must be greater than 0).", optarg);
+                bye("Invalid -d delay \"%s\" (must be greater than 0).", optarg);
             if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
                 bye("Since April 2010, the default unit for -d is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
             break;
@@ -358,8 +419,8 @@ int main(int argc, char *argv[])
             o.idletimeout = tval2msecs(optarg);
             if (o.idletimeout <= 0)
                 bye("Invalid -i timeout (must be greater than 0).");
-            if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
-                bye("Since April 2010, the default unit for -i is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
+            if (o.idletimeout >= 100 * 1000 && tval_unit(optarg) == NULL)
+                bye("Since April 2010, the default unit for -i is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.idletimeout / 1000.0 / 60, optarg, o.idletimeout / 1000.0);
             break;
         case 's':
             source = optarg;
@@ -368,7 +429,7 @@ int main(int argc, char *argv[])
             o.listen = 1;
             break;
         case 'u':
-            o.udp = 1;
+            o.proto = IPPROTO_UDP;
             break;
         case 'v':
             /* One -v activites verbose, after that it's debugging. */
@@ -384,8 +445,8 @@ int main(int argc, char *argv[])
             o.conntimeout = tval2msecs(optarg);
             if (o.conntimeout <= 0)
                 bye("Invalid -w timeout (must be greater than 0).");
-            if (o.linedelay >= 100 * 1000 && tval_unit(optarg) == NULL)
-                bye("Since April 2010, the default unit for -w is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.linedelay / 1000.0 / 60, optarg, o.linedelay / 1000.0);
+            if (o.conntimeout >= 100 * 1000 && tval_unit(optarg) == NULL)
+                bye("Since April 2010, the default unit for -w is seconds, so your time of \"%s\" is %.1f minutes. Use \"%sms\" for %g milliseconds.", optarg, o.conntimeout / 1000.0 / 60, optarg, o.conntimeout / 1000.0);
             break;
         case 't':
             o.telnet = 1;
@@ -394,83 +455,58 @@ int main(int argc, char *argv[])
             if (strcmp(long_options[option_index].name, "version") == 0) {
                 print_banner();
                 exit(EXIT_SUCCESS);
-            }
-            else if (strcmp(long_options[option_index].name, "proxy") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "proxy") == 0) {
                 if (proxyaddr)
                     bye("You can't specify more than one --proxy.");
                 proxyaddr = Strdup(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "proxy-type") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "proxy-type") == 0) {
                 if (o.proxytype)
-                        bye("You can't specify more than one --proxy-type.");
+                    bye("You can't specify more than one --proxy-type.");
                 o.proxytype = Strdup(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "proxy-auth") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "proxy-auth") == 0) {
                 if (o.proxy_auth)
-                        bye("You can't specify more than one --proxy-auth.");
+                    bye("You can't specify more than one --proxy-auth.");
                 o.proxy_auth = Strdup(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "nsock-engine") == 0)
-            {
-                nsock_set_default_engine(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "broker") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "nsock-engine") == 0) {
+                if (nsock_set_default_engine(optarg) < 0)
+                    bye("Unknown or non-available engine: %s.", optarg);
+                o.nsock_engine = 1;
+            } else if (strcmp(long_options[option_index].name, "broker") == 0) {
                 o.broker = 1;
                 /* --broker implies --listen. */
                 o.listen = 1;
-            }
-            else if (strcmp(long_options[option_index].name, "chat") == 0
-                     || strcmp(long_options[option_index].name, "talk") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "chat") == 0
+                       || strcmp(long_options[option_index].name, "talk") == 0) {
                 /* --talk is an older name for --chat. */
                 o.chat = 1;
                 /* --chat implies --broker. */
                 o.broker = 1;
-            }
-            else if (strcmp(long_options[option_index].name, "allow") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "allow") == 0) {
                 o.allow = 1;
                 host_list_add_spec(&allow_host_list, optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "allowfile") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "allowfile") == 0) {
                 o.allow = 1;
                 host_list_add_filename(&allow_host_list, optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "deny") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "deny") == 0) {
                 host_list_add_spec(&deny_host_list, optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "denyfile") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "denyfile") == 0) {
                 host_list_add_filename(&deny_host_list, optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "append-output") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "append-output") == 0) {
                 o.append = 1;
+            } else if (strcmp(long_options[option_index].name, "sctp") == 0) {
+                o.proto = IPPROTO_SCTP;
             }
 #ifdef HAVE_OPENSSL
-            else if (strcmp(long_options[option_index].name, "ssl-cert") == 0)
-            {
+            else if (strcmp(long_options[option_index].name, "ssl-cert") == 0) {
                 o.ssl = 1;
                 o.sslcert = Strdup(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "ssl-key") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "ssl-key") == 0) {
                 o.ssl = 1;
                 o.sslkey = Strdup(optarg);
-            }
-            else if (strcmp(long_options[option_index].name, "ssl-verify") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "ssl-verify") == 0) {
                 o.sslverify = 1;
                 o.ssl = 1;
-            }
-            else if (strcmp(long_options[option_index].name, "ssl-trustfile") == 0)
-            {
+            } else if (strcmp(long_options[option_index].name, "ssl-trustfile") == 0) {
                 o.ssl = 1;
                 if (o.ssltrustfile != NULL)
                     bye("The --ssl-trustfile option may be given only once.");
@@ -479,6 +515,35 @@ int main(int argc, char *argv[])
                    verification. */
                 o.sslverify = 1;
             }
+#endif
+#ifdef HAVE_LUA
+            else if (strcmp(long_options[option_index].name, "lua-exec") == 0) {
+                if (o.cmdexec != NULL)
+                    bye("Only one of --exec, --sh-exec, and --lua-exec is allowed.");
+                o.cmdexec = optarg;
+                o.execmode = EXEC_LUA;
+            } else if (strcmp(long_options[option_index].name, "lua-exec-internal") == 0) {
+                /* This command-line switch is undocumented on purpose. Do NOT use it
+                   explicitly as its behavior might differ between Ncat releases.
+
+                   Its goal is to switch the Ncat process to the Lua interpreter state
+                   so that its standard output and input can be redirected to
+                   particular connection's streams. Although it is implemented by
+                   forking in POSIX builds, Windows does not have the fork() system
+                   call and thus requires this workaround. More info here:
+                   http://seclists.org/nmap-dev/2013/q2/492 */
+#ifdef WIN32
+                if (o.debug)
+                    logdebug("Enabling binary stdout for the Lua output.\n");
+                int result = _setmode(_fileno(stdout), _O_BINARY);
+                if (result == -1)
+                    perror("Cannot set mode");
+#endif
+                ncat_assert(argc == 3);
+                o.cmdexec = argv[2];
+                lua_setup();
+                lua_run();
+            }
 #endif
             break;
         case 'h':
@@ -490,9 +555,15 @@ int main(int argc, char *argv[])
 "'s' for seconds, 'm' for minutes, or 'h' for hours (e.g. 500ms).\n"
 "  -4                         Use IPv4 only\n"
 "  -6                         Use IPv6 only\n"
+#if HAVE_SYS_UN_H
+"  -U, --unixsock             Use Unix domain sockets only\n"
+#endif
 "  -C, --crlf                 Use CRLF for EOL sequence\n"
 "  -c, --sh-exec <command>    Executes the given command via /bin/sh\n"
 "  -e, --exec <command>       Executes the given command\n"
+#ifdef HAVE_LUA
+"      --lua-exec <filename>  Executes the given Lua script\n"
+#endif
 "  -g hop1[,hop2,...]         Loose source routing hop points (8 max)\n"
 "  -G <n>                     Loose source routing hop pointer (4, 8, 12, ...)\n"
 "  -m, --max-conns <n>        Maximum <n> simultaneous connections\n"
@@ -509,7 +580,7 @@ int main(int argc, char *argv[])
 "  -t, --telnet               Answer Telnet negotiations\n"
 "  -u, --udp                  Use UDP instead of default TCP\n"
 "      --sctp                 Use SCTP instead of default TCP\n"
-"  -v, --verbose              Set verbosity level (can be used up to 3 times)\n"
+"  -v, --verbose              Set verbosity level (can be used several times)\n"
 "  -w, --wait <time>          Connect timeout\n"
 "      --append-output        Append rather than clobber specified output files\n"
 "      --send-only            Only send data, ignoring received; quit on EOF\n"
@@ -557,7 +628,25 @@ int main(int argc, char *argv[])
     else
         nbase_set_log(loguser, NULL);
 
-    /* Will be AF_INET or AF_INET6 when valid */
+#if HAVE_SYS_UN_H
+    /* Using Unix domain sockets, so do the checks now */
+    if (o.af == AF_UNIX) {
+        if (proxyaddr || o.proxytype)
+            bye("Proxy option not supported when using Unix domain sockets.");
+#ifdef HAVE_OPENSSL
+        if (o.ssl)
+            bye("SSL option not supported when using Unix domain sockets.");
+#endif
+        if (o.broker)
+            bye("Connection brokering not supported when using Unix domain sockets.");
+        if (srcport != -1)
+            bye("Specifying source port when using Unix domain sockets doesn't make sense.");
+        if (o.numsrcrtes > 0)
+            bye("Loose source routing not allowed when using Unix domain sockets.");
+    }
+#endif  /* HAVE_SYS_UN_H */
+
+    /* Will be AF_INET or AF_INET6 or AF_UNIX when valid */
     memset(&targetss.storage, 0, sizeof(targetss.storage));
     targetss.storage.ss_family = AF_UNSPEC;
     httpconnect.storage = socksconnect.storage = srcaddr.storage = targetss.storage;
@@ -569,34 +658,34 @@ int main(int argc, char *argv[])
     }
 
     if (proxyaddr) {
-      if (!o.proxytype)
-          o.proxytype = Strdup("http");
+        if (!o.proxytype)
+            o.proxytype = Strdup("http");
 
-      if (!strcmp(o.proxytype, "http")) {
-          /* Parse HTTP proxy address and temporarily store it in httpconnect.  If
-           * the proxy server is given as an IPv6 address (not hostname), the port
-           * number MUST be specified as well or parsing will break (due to the
-           * colons in the IPv6 address and host:port separator).
-           */
+        if (!strcmp(o.proxytype, "http")) {
+            /* Parse HTTP proxy address and temporarily store it in httpconnect.  If
+             * the proxy server is given as an IPv6 address (not hostname), the port
+             * number MUST be specified as well or parsing will break (due to the
+             * colons in the IPv6 address and host:port separator).
+             */
 
-          parseproxy(proxyaddr, &httpconnect.storage, DEFAULT_PROXY_PORT);
-      } else if (!strcmp(o.proxytype, "socks4") || !strcmp(o.proxytype, "4")) {
-          /* Parse SOCKS proxy address and temporarily store it in socksconnect */
+            httpconnectlen = parseproxy(proxyaddr, &httpconnect.storage, DEFAULT_PROXY_PORT);
+        } else if (!strcmp(o.proxytype, "socks4") || !strcmp(o.proxytype, "4")) {
+            /* Parse SOCKS proxy address and temporarily store it in socksconnect */
 
-          parseproxy(proxyaddr, &socksconnect.storage, DEFAULT_SOCKS4_PORT);
-      } else {
-          bye("Invalid proxy type \"%s\".", o.proxytype);
-      }
+            socksconnectlen = parseproxy(proxyaddr, &socksconnect.storage, DEFAULT_SOCKS4_PORT);
+        } else {
+            bye("Invalid proxy type \"%s\".", o.proxytype);
+        }
 
-      free(o.proxytype);
-      free(proxyaddr);
+        free(o.proxytype);
+        free(proxyaddr);
     } else {
-      if (o.proxytype) {
-        if (!o.listen)
-          bye("Proxy type (--proxy-type) specified without proxy address (--proxy).");
-        if (strcmp(o.proxytype, "http"))
-          bye("Invalid proxy type \"%s\".", o.proxytype);
-      }
+        if (o.proxytype) {
+            if (!o.listen)
+                bye("Proxy type (--proxy-type) specified without proxy address (--proxy).");
+            if (strcmp(o.proxytype, "http"))
+                bye("Invalid proxy type \"%s\".", o.proxytype);
+        }
     }
 
     /* Default port */
@@ -604,11 +693,28 @@ int main(int argc, char *argv[])
 
     /* Resolve the given source address */
     if (source) {
+        int rc = 0;
+
         if (o.listen)
             bye("-l and -s are incompatible.  Specify the address and port to bind to like you would a host to connect to.");
 
-        if (!resolve(source, 0, &srcaddr.storage, &srcaddrlen, o.af))
-            bye("Could not resolve source address %s.", source);
+#if HAVE_SYS_UN_H
+        /* if using UNIX sockets just copy the path.
+         * If it's not valid, it will fail later! */
+        if (o.af == AF_UNIX) {
+            if (o.proto == IPPROTO_UDP) {
+                srcaddr.un.sun_family = AF_UNIX;
+                strncpy(srcaddr.un.sun_path, source, sizeof(srcaddr.un.sun_path));
+                srcaddrlen = SUN_LEN(&srcaddr.un);
+            }
+            else
+                if (o.verbose)
+                    loguser("Specifying source socket for other than DATAGRAM Unix domain sockets have no effect.\n");
+        } else
+#endif
+            rc = resolve(source, 0, &srcaddr.storage, &srcaddrlen, o.af);
+        if (rc != 0)
+            bye("Could not resolve source address \"%s\": %s.", source, gai_strerror(rc));
     }
 
     host_list_to_set(&o.allowset, allow_host_list);
@@ -617,16 +723,37 @@ int main(int argc, char *argv[])
     host_list_free(deny_host_list);
 
     if (optind == argc) {
+#if HAVE_SYS_UN_H
+        if (o.af == AF_UNIX) {
+            if (!o.listen)
+                bye("You have to specify path to a socket to connect to.");
+            else
+                bye("You have to specify path to a socket to listen on.");
+        }
+#endif
         /* Listen defaults to any address and DEFAULT_NCAT_PORT */
         if (!o.listen)
             bye("You must specify a host to connect to.");
     } else {
+#if HAVE_SYS_UN_H
+        if (o.af == AF_UNIX) {
+            memset(&targetss.storage, 0, sizeof(struct sockaddr_un));
+            targetss.un.sun_family = AF_UNIX;
+            strncpy(targetss.un.sun_path, argv[optind], sizeof(targetss.un.sun_path));
+            targetsslen = SUN_LEN(&targetss.un);
+            o.target = argv[optind];
+            optind++;
+        } else
+#endif
         /* Resolve hostname if we're given one */
         if (strspn(argv[optind], "0123456789") != strlen(argv[optind])) {
+            int rc;
+
             o.target = argv[optind];
             /* resolve hostname */
-            if (!resolve(o.target, 0, &targetss.storage, &targetsslen, o.af))
-                bye("Could not resolve hostname %s.", o.target);
+            rc = resolve(o.target, 0, &targetss.storage, &targetsslen, o.af);
+            if (rc != 0)
+                bye("Could not resolve hostname \"%s\": %s.", o.target, gai_strerror(rc));
             optind++;
         } else {
             if (!o.listen)
@@ -635,6 +762,12 @@ int main(int argc, char *argv[])
     }
 
     /* Whatever's left is the port number; there should be at most one. */
+#if HAVE_SYS_UN_H
+    /* We do not use ports with Unix domain sockets. */
+    if (o.af == AF_UNIX && optind > argc)
+        bye("Using Unix domain sockets and specifying port doesn't make sense.");
+#endif
+
     if (optind + 1 < argc || (o.listen && srcport != -1 && optind + 1 == argc)) {
         loguser("Got more than one port specification:");
         if (o.listen && srcport != -1)
@@ -654,12 +787,21 @@ int main(int argc, char *argv[])
         o.portno = (unsigned short) long_port;
     }
 
-    if (o.af == AF_INET)
+    if (targetss.storage.ss_family == AF_INET)
         targetss.in.sin_port = htons(o.portno);
 #ifdef HAVE_IPV6
-    else
+    else if (targetss.storage.ss_family == AF_INET6)
         targetss.in6.sin6_port = htons(o.portno);
 #endif
+#if HAVE_SYS_UN_H
+    /* If we use Unix domain sockets, we have to count with them. */
+    else if (targetss.storage.ss_family == AF_UNIX)
+        ; /* Do nothing. */
+#endif
+    else if (targetss.storage.ss_family == AF_UNSPEC)
+        ; /* Leave unspecified. */
+    else
+        bye("Unknown address family %d.", targetss.storage.ss_family);
 
     if (srcport != -1) {
         if (o.listen) {
@@ -691,15 +833,21 @@ int main(int argc, char *argv[])
      */
     if (httpconnect.storage.ss_family != AF_UNSPEC) {
         union sockaddr_u tmp = targetss;
+        size_t tmplen = targetsslen;
         targetss = httpconnect;
+        targetsslen = httpconnectlen;
         httpconnect = tmp;
+        httpconnectlen = tmplen;
     } else if (socksconnect.storage.ss_family != AF_UNSPEC) {
         union sockaddr_u tmp = targetss;
+        size_t tmplen = targetsslen;
         targetss = socksconnect;
+        targetsslen = socksconnectlen;
         socksconnect = tmp;
+        socksconnectlen = tmplen;
     }
 
-    if (o.udp) {
+    if (o.proto == IPPROTO_UDP) {
         /* Don't allow a false sense of security if someone tries SSL over UDP. */
         if (o.ssl)
             bye("UDP mode does not support SSL.");
@@ -707,7 +855,7 @@ int main(int argc, char *argv[])
             bye("UDP mode does not support the -k or --keep-open options, except with --exec or --sh-exec.");
         if (o.broker)
             bye("UDP mode does not support connection brokering.\n\
-If this feature is important to you, write nmap-dev@insecure.org with a\n\
+If this feature is important to you, write dev@nmap.org with a\n\
 description of how you intend to use it, as an aid to deciding how UDP\n\
 connection brokering should work.");
     }
@@ -716,6 +864,11 @@ connection brokering should work.");
        the console. A no-op on Unix. */
     set_lf_mode();
 
+#ifdef HAVE_LUA
+    if (o.execmode == EXEC_LUA)
+        lua_setup();
+#endif
+
     if (o.listen)
         return ncat_listen_mode();
     else
@@ -723,7 +876,8 @@ connection brokering should work.");
 }
 
 /* connect error handling and operations. */
-static int ncat_connect_mode(void) {
+static int ncat_connect_mode(void)
+{
     /*
      * allow/deny commands with connect make no sense. If you don't want to
      * connect to a host, don't try to.
@@ -744,14 +898,12 @@ static int ncat_connect_mode(void) {
     return ncat_connect();
 }
 
-static int ncat_listen_mode(void) {
+static int ncat_listen_mode(void)
+{
     /* Can't 'listen' AND 'connect' to a proxy server at the same time. */
     if (httpconnect.storage.ss_family != AF_UNSPEC || socksconnect.storage.ss_family != AF_UNSPEC)
         bye("Invalid option combination: --proxy and -l.");
 
-    if (o.idletimeout != 0)
-        bye("An idle timeout only works in connect mode.");
-
     if (o.broker && o.cmdexec != NULL)
         bye("Invalid option combination: --broker and -e.");
 
@@ -768,7 +920,7 @@ static int ncat_listen_mode(void) {
 
 #ifndef WIN32
     /* See if the shell is executable before we get deep into this */
-    if (o.shellexec && access("/bin/sh", X_OK) == -1)
+    if (o.execmode == EXEC_SHELL && access("/bin/sh", X_OK) == -1)
         bye("/bin/sh is not executable, so `-c' won't work.");
 #endif
 
@@ -779,22 +931,22 @@ static int ncat_listen_mode(void) {
         int rc;
 
         /* No command-line address. Listen on IPv4 or IPv6 or both. */
-	/* Try to bind to IPv6 first; on AIX a bound IPv4 socket blocks an IPv6
+        /* Try to bind to IPv6 first; on AIX a bound IPv4 socket blocks an IPv6
            socket on the same port, despite IPV6_V6ONLY. */
 #ifdef HAVE_IPV6
         if (o.af == AF_INET6 || o.af == AF_UNSPEC) {
             ss_len = sizeof(listenaddrs[num_listenaddrs]);
             rc = resolve("::", o.portno, &listenaddrs[num_listenaddrs].storage, &ss_len, AF_INET6);
-            if (!rc)
-                bye("Failed to resolve default IPv6 address.");
+            if (rc != 0)
+                bye("Failed to resolve default IPv6 address: %s.", gai_strerror(rc));
             num_listenaddrs++;
         }
 #endif
         if (o.af == AF_INET || o.af == AF_UNSPEC) {
             ss_len = sizeof(listenaddrs[num_listenaddrs]);
             rc = resolve("0.0.0.0", o.portno, &listenaddrs[num_listenaddrs].storage, &ss_len, AF_INET);
-            if (!rc)
-                bye("Failed to resolve default IPv4 address.");
+            if (rc != 0)
+                bye("Failed to resolve default IPv4 address: %s.", gai_strerror(rc));
             num_listenaddrs++;
         }
     }
