@@ -3,12 +3,6 @@
 
 #define BITLIBNAME "bit"
 
-extern "C" {
-#include "lauxlib.h"
-#include "lua.h"
-}
-
 LUALIB_API int luaopen_bit(lua_State *L);
 
 #endif
-
