@@ -24,15 +24,17 @@
 #include <sstream>
 #include <iomanip>
 
-#define NMAP_NSOCK_SOCKET  "NMAP_NSOCK_SOCKET"
-#define NMAP_NSOCK_PCAP_SOCKET  "NMAP_NSOCK_PCAP_SOCKET"
-
 #define DEFAULT_TIMEOUT 30000
 
-/* Integer keys in Nsock function environments */
-#define THREAD_SOCKETS     1           /* <Thread, Table of Sockets (keys)> */
-#define CONNECT_WAITING    2           /* Threads waiting to lock */
-#define KEY_PCAP           3           /* keys to pcap sockets */
+/* Upvalues for library variables */
+enum {
+  NSOCK_POOL = lua_upvalueindex(1),
+  NSOCK_SOCKET = lua_upvalueindex(2), /* nsock socket metatable */
+  PCAP_SOCKET = lua_upvalueindex(3), /* pcap socket metatable */
+  THREAD_SOCKETS = lua_upvalueindex(4), /* <Thread, Table of Sockets (keys)> */
+  CONNECT_WAITING = lua_upvalueindex(5), /* Threads waiting to lock */
+  KEY_PCAP = lua_upvalueindex(6) /* Keys to pcap sockets */
+};
 
 /* Integer keys in the Nsock userdata environments */
 #define THREAD_I  1 /* The thread that yielded */
@@ -65,8 +67,6 @@ typedef struct nse_nsock_udata
 
 } nse_nsock_udata;
 
-static int NSOCK_POOL = 0xac1dba11;
-
 static int gc_pool (lua_State *L)
 {
   nsock_pool *nsp = (nsock_pool *) lua_touserdata(L, 1);
@@ -80,27 +80,34 @@ static nsock_pool new_pool (lua_State *L)
 {
   nsock_pool nsp = nsp_new(NULL);
   nsock_pool *nspp;
+
+  /* configure logging */
+  nsock_set_log_function(nsp, nmap_nsock_stderr_logger);
+  nmap_adjust_loglevel(nsp, o.scriptTrace());
+
+  nsp_setdevice(nsp, o.device);
+
+  if (o.proxy_chain)
+    nsp_set_proxychain(nsp, o.proxy_chain);
+
   nsp_setbroadcast(nsp, true);
-  lua_pushlightuserdata(L, &NSOCK_POOL);
+
   nspp = (nsock_pool *) lua_newuserdata(L, sizeof(nsock_pool));
   *nspp = nsp;
   lua_newtable(L);
   lua_pushcfunction(L, gc_pool);
   lua_setfield(L, -2, "__gc");
   lua_setmetatable(L, -2);
-  lua_rawset(L, LUA_REGISTRYINDEX);
   return nsp;
 }
 
 static nsock_pool get_pool (lua_State *L)
 {
-  nsock_pool *nsp;
-  lua_pushlightuserdata(L, &NSOCK_POOL);
-  lua_rawget(L, LUA_REGISTRYINDEX);
-  nsp = (nsock_pool *) lua_touserdata(L, -1);
-  assert(*nsp != NULL);
-  lua_pop(L, 1);
-  return *nsp;
+  nsock_pool *nspp;
+  nspp = (nsock_pool *) lua_touserdata(L, NSOCK_POOL);
+  assert(nspp != NULL);
+  assert(*nspp != NULL);
+  return *nspp;
 }
 
 static std::string hexify (const unsigned char *str, size_t len)
@@ -143,7 +150,7 @@ static std::string hexify (const unsigned char *str, size_t len)
 
 /* Some constants used for enforcing a limit on the number of open sockets
  * in use by threads. The maximum value between MAX_PARALLELISM and
- * o.maxparallelism is the max # of threads that can have connected sockets
+ * o.max_parallelism is the max # of threads that can have connected sockets
  * (open).
  *
  * THREAD_SOCKETS is a weak keyed table of <Thread, Socket Table> pairs.
@@ -157,21 +164,18 @@ static std::string hexify (const unsigned char *str, size_t len)
  */
 #define MAX_PARALLELISM   20
 
-/* The Lua 5.2 socket_lock function */
-#if 0
 /* int socket_lock (lua_State *L)
  *
  * This function is called by l_connect to get a "lock" on a socket.
  * When connect calls this function, it expects socket_lock to yield forcing
  * connect to be restarted when resumed or it succeeds returning normally.
  */
-static void socket_lock (lua_State *L, int idx)
+static int socket_lock (lua_State *L, int idx)
 {
   unsigned p = o.max_parallelism == 0 ? MAX_PARALLELISM : o.max_parallelism;
   int top = lua_gettop(L);
-  lua_rawgeti(L, LUA_ENVIRONINDEX, THREAD_SOCKETS);
   nse_base(L);
-  lua_rawget(L, -2);
+  lua_rawget(L, THREAD_SOCKETS);
   if (lua_istable(L, -1))
   {
     /* Thread already has a "lock" with open sockets. Place the new socket
@@ -179,74 +183,25 @@ static void socket_lock (lua_State *L, int idx)
     lua_pushvalue(L, idx);
     lua_pushboolean(L, true);
     lua_rawset(L, -3);
-  } else if (table_length(L, top+2) <= p)
+  } else if (nseU_tablen(L, THREAD_SOCKETS) <= p)
   {
     /* There is room for this thread to open sockets */
     nse_base(L);
-    weak_table(L, 0, 0, "k"); /* weak socket references */
+    nseU_weaktable(L, 0, 0, "k"); /* weak socket references */
     lua_pushvalue(L, idx); /* socket */
     lua_pushboolean(L, true);
     lua_rawset(L, -3); /* add to sockets table */
-    lua_rawset(L, top+2); /* add new <Thread, Sockets Table> Pair
-                       * to THREAD_SOCKETS */
+    lua_rawset(L, THREAD_SOCKETS); /* add new <Thread, Sockets Table> Pair
+                                    * to THREAD_SOCKETS */
   } else
   {
-    /* Too many threads have sockets open. Add thread to waiting. The caller
-     * is expected to yield. (see the connect function in luaopen_nsock) */
-    lua_rawgeti(L, LUA_ENVIRONINDEX, CONNECT_WAITING);
     nse_base(L);
     lua_pushboolean(L, true);
-    lua_settable(L, -3);
+    lua_rawset(L, CONNECT_WAITING);
     lua_settop(L, top); /* restore stack to original condition for l_connect */
-    return nse_yield(L, 0, NULL);
+    return 0;
   }
   lua_settop(L, top); /* restore stack to original condition for l_connect */
-}
-#endif
-
-/* int socket_lock (lua_State *L)
- *
- * Arguments
- *   socket  A socket to "lock".
- *
- * This function is called by Lua to get a "lock" on a socket.
- * See the connect function (in Lua) in luaopen_nsock.
- */
-static int socket_lock (lua_State *L)
-{
-  unsigned p = o.max_parallelism == 0 ? MAX_PARALLELISM : o.max_parallelism;
-  lua_settop(L, 1);
-  lua_rawgeti(L, LUA_ENVIRONINDEX, THREAD_SOCKETS);
-  nse_base(L);
-  lua_rawget(L, -2);
-  if (lua_istable(L, -1))
-  {
-    /* Thread already has a "lock" with open sockets. Place the new socket
-     * in its sockets table */
-    lua_pushvalue(L, 1);
-    lua_pushboolean(L, true);
-    lua_rawset(L, -3);
-  } else if (table_length(L, 2) <= p)
-  {
-    /* There is room for this thread to open sockets */
-    nse_base(L);
-    weak_table(L, 0, 0, "k"); /* weak socket references */
-    lua_pushvalue(L, 1); /* socket */
-    lua_pushboolean(L, true);
-    lua_rawset(L, -3); /* add to sockets table */
-    lua_rawset(L, 2); /* add new <Thread, Sockets Table> Pair
-                       * to THREAD_SOCKETS */
-  } else
-  {
-    /* Too many threads have sockets open. Add thread to waiting. The caller
-     * is expected to yield. (see the connect function in luaopen_nsock) */
-    lua_rawgeti(L, LUA_ENVIRONINDEX, CONNECT_WAITING);
-    nse_base(L);
-    lua_pushboolean(L, true);
-    lua_settable(L, -3);
-    return nse_yield(L, 0, NULL);
-  }
-  lua_pushboolean(L, true);
   return 1;
 }
 
@@ -256,19 +211,15 @@ static void socket_unlock (lua_State *L)
 
   lua_gc(L, LUA_GCSTOP, 0); /* don't collect threads during iteration */
 
-  lua_rawgeti(L, LUA_ENVIRONINDEX, THREAD_SOCKETS);
-  lua_pushnil(L);
-  while (lua_next(L, -2) != 0)
+  for (lua_pushnil(L); lua_next(L, THREAD_SOCKETS); lua_pop(L, 1))
   {
     unsigned open = 0;
 
     if (lua_status(lua_tothread(L, -2)) == LUA_YIELD)
     {
-      lua_pushnil(L);
-      while (lua_next(L, -2) != 0) /* for each socket */
+      for (lua_pushnil(L); lua_next(L, -2); lua_pop(L, 1)) /* for each socket */
       {
-        lua_pop(L, 1); /* pop garbage boolean */
-        if (((nse_nsock_udata *) lua_touserdata(L, -1))->nsiod != NULL)
+        if (((nse_nsock_udata *) lua_touserdata(L, -2))->nsiod != NULL)
           open++;
       }
     }
@@ -276,33 +227,25 @@ static void socket_unlock (lua_State *L)
     if (open == 0) /* thread has no open sockets? */
     {
       /* close all of its sockets */
-      lua_pushnil(L);
-      while (lua_next(L, -2) != 0) /* for each socket */
+      for (lua_pushnil(L); lua_next(L, -2); lua_pop(L, 1)) /* for each socket */
       {
-        lua_pop(L, 1); /* pop garbage boolean */
-        lua_getfield(L, -1, "close");
-        lua_pushvalue(L, -2);
+        lua_getfield(L, -2, "close");
+        lua_pushvalue(L, -3);
         lua_call(L, 1, 0);
       }
 
       lua_pushvalue(L, -2); /* thread key */
       lua_pushnil(L);
-      lua_rawset(L, top+1); /* THREADS_SOCKETS */
+      lua_rawset(L, THREAD_SOCKETS);
 
-      lua_rawgeti(L, LUA_ENVIRONINDEX, CONNECT_WAITING);
-      lua_pushnil(L);
-      while (lua_next(L, -2) != 0)
+      for (lua_pushnil(L); lua_next(L, CONNECT_WAITING); lua_pop(L, 1))
       {
-        lua_pop(L, 1); /* pop garbage boolean */
-        nse_restore(lua_tothread(L, -1), 0);
-        lua_pushvalue(L, -1);
+        nse_restore(lua_tothread(L, -2), 0);
+        lua_pushvalue(L, -2);
         lua_pushnil(L);
-        lua_rawset(L, -4); /* remove thread from waiting */
+        lua_rawset(L, CONNECT_WAITING);
       }
-      lua_pop(L, 1); /* CONNECT_WAITING */
     }
-
-    lua_pop(L, 1); /* pop sockets table */
   }
 
   lua_gc(L, LUA_GCRESTART, 0);
@@ -392,6 +335,7 @@ static void status (lua_State *L, enum nse_status status)
     case NSE_STATUS_EOF:
     case NSE_STATUS_ERROR:
     case NSE_STATUS_TIMEOUT:
+    case NSE_STATUS_PROXYERROR:
       lua_pushnil(L);
       lua_pushstring(L, nse_status2str(status));
       nse_restore(L, 2);
@@ -415,7 +359,7 @@ static void callback (nsock_pool nsp, nsock_event nse, void *ud)
 static int yield (lua_State *L, nse_nsock_udata *nu, const char *action,
     const char *direction, int ctx, lua_CFunction k)
 {
-  lua_getfenv(L, 1);
+  lua_getuservalue(L, 1);
   lua_pushthread(L);
   lua_rawseti(L, -2, THREAD_I);
   lua_pop(L, 1); /* nsock udata environment */
@@ -425,14 +369,15 @@ static int yield (lua_State *L, nse_nsock_udata *nu, const char *action,
   return nse_yield(L, ctx, k);
 }
 
-static nse_nsock_udata *check_nsock_udata (lua_State *L, int idx, int open)
+/* In the case of unconnected UDP sockets, this function will call
+   nsock_setup_udp on your behalf before returning true. */
+static nse_nsock_udata *check_nsock_udata (lua_State *L, int idx, bool open)
 {
-  nse_nsock_udata *nu =
-      (nse_nsock_udata *) luaL_checkudata(L, idx, NMAP_NSOCK_SOCKET);
+  nse_nsock_udata *nu = (nse_nsock_udata *) nseU_checkudata(L, idx, NSOCK_SOCKET, "nsock");
 
   if (open && nu->nsiod == NULL) {
     /* The socket hasn't been connected or setup yet. Try doing a setup, or
-       throw and error if that's not possible. */
+       throw an error if that's not possible. */
     if (nu->proto == IPPROTO_UDP) {
       nsock_pool nsp;
 
@@ -453,35 +398,41 @@ static nse_nsock_udata *check_nsock_udata (lua_State *L, int idx, int open)
         luaL_error(L, "Error in setup of iod with proto %d and af %d: %s (%d)",
           nu->proto, nu->af, socket_strerror(socket_errno()), socket_errno());
       }
-    } else {
-      luaL_error(L, "socket must be connected\n");
     }
   }
 
   return nu;
 }
 
-static int loop (lua_State *L)
+/* If the socket udata nu is not open, return from the enclosing function with
+   an error. */
+#define NSOCK_UDATA_ENSURE_OPEN(L, nu) \
+do { \
+  if (nu->nsiod == NULL) \
+    return nseU_safeerror(L, "socket must be connected"); \
+} while (0)
+
+static int l_loop (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
   int tout = luaL_checkint(L, 1);
 
-  /* clean up old socket locks */
-  socket_unlock(L);
+  socket_unlock(L); /* clean up old socket locks */
 
+  nmap_adjust_loglevel(nsp, o.scriptTrace());
   if (nsock_loop(nsp, tout) == NSOCK_LOOP_ERROR)
-    luaL_error(L, "a fatal error occurred in nsock_loop");
+    return luaL_error(L, "a fatal error occurred in nsock_loop");
   return 0;
 }
 
 static int l_reconnect_ssl (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
 
 #ifndef HAVE_OPENSSL
-  if (1)
-    return safe_error(L, "sorry, you don't have OpenSSL");
+  return nseU_safeerror(L, "sorry, you don't have OpenSSL");
 #endif
 
   nsock_reconnect_ssl(nsp, nu->nsiod, callback, nu->timeout,
@@ -490,16 +441,18 @@ static int l_reconnect_ssl (lua_State *L)
   return yield(L, nu, "SSL RECONNECT", TO, 0, NULL);
 }
 
+static void close_internal (lua_State *L, nse_nsock_udata *nu);
+
 static int l_connect (lua_State *L)
 {
   enum type {TCP, UDP, SSL};
   static const char * const op[] = {"tcp", "udp", "ssl", NULL};
 
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
-  const char *addr, *targetname; check_target(L, 2, &addr, &targetname);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
+  const char *addr, *targetname; nseU_checktarget(L, 2, &addr, &targetname);
   const char *default_proto = NULL;
-  unsigned short port = check_port(L, 3, &default_proto);
+  unsigned short port = nseU_checkport(L, 3, &default_proto);
   if (default_proto == NULL) {
     switch (nu->proto) {
     case IPPROTO_TCP:
@@ -517,26 +470,23 @@ static int l_connect (lua_State *L)
   struct addrinfo *dest;
   int error_id;
 
-  /* Lua 5.2 */
-#if 0
-  /* either socket_lock yields and this function is resumed (and restarted)
-   * or it succeeds and we continue.
-   */
-  socket_lock(L);
-#endif
+  if (!socket_lock(L, 1)) /* we cannot get a socket lock */
+    return nse_yield(L, 0, l_connect); /* restart on continuation */
 
 #ifndef HAVE_OPENSSL
   if (what == SSL)
-    return safe_error(L, "sorry, you don't have OpenSSL");
+    return nseU_safeerror(L, "sorry, you don't have OpenSSL");
 #endif
 
   error_id = getaddrinfo(addr, NULL, NULL, &dest);
   if (error_id)
-    return safe_error(L, gai_strerror(error_id));
+    return nseU_safeerror(L, gai_strerror(error_id));
 
   if (dest == NULL)
-    return safe_error(L, "getaddrinfo returned success but no addresses");
+    return nseU_safeerror(L, "getaddrinfo returned success but no addresses");
 
+  if (nu->nsiod != NULL)
+    close_internal(L, nu);
   nu->nsiod = nsi_new(nsp, NULL);
   if (nu->source_addr.ss_family != AF_UNSPEC) {
     nsi_set_localaddr(nu->nsiod, &nu->source_addr, nu->source_addrlen);
@@ -583,7 +533,8 @@ static int l_connect (lua_State *L)
 static int l_send (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   size_t size;
   const char *string = luaL_checklstring(L, 2, &size);
   trace(nu->nsiod, hexify((unsigned char *) string, size).c_str(), TO);
@@ -594,24 +545,26 @@ static int l_send (lua_State *L)
 static int l_sendto (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   size_t size;
-  const char *addr, *targetname; check_target(L, 2, &addr, &targetname);
+  const char *addr, *targetname; nseU_checktarget(L, 2, &addr, &targetname);
   const char *default_proto = NULL;
-  unsigned short port = check_port(L, 3, &default_proto);
+  unsigned short port = nseU_checkport(L, 3, &default_proto);
   const char *string = luaL_checklstring(L, 4, &size);
   int error_id;
   struct addrinfo *dest;
 
   error_id = getaddrinfo(addr, NULL, NULL, &dest);
   if (error_id)
-    return safe_error(L, gai_strerror(error_id));
+    return nseU_safeerror(L, gai_strerror(error_id));
 
   if (dest == NULL)
-    return safe_error(L, "getaddrinfo returned success but no addresses");
+    return nseU_safeerror(L, "getaddrinfo returned success but no addresses");
 
   nsock_sendto(nsp, nu->nsiod, callback, nu->timeout, nu, dest->ai_addr, dest->ai_addrlen, port, string, size);
   trace(nu->nsiod, hexify((unsigned char *) string, size).c_str(), TO);
+  freeaddrinfo(dest);
   return yield(L, nu, "SEND", TO, 0, NULL);
 	
 }
@@ -637,7 +590,8 @@ static void receive_callback (nsock_pool nsp, nsock_event nse, void *udata)
 static int l_receive (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   nsock_read(nsp, nu->nsiod, receive_callback, nu->timeout, nu);
   return yield(L, nu, "RECEIVE", FROM, 0, NULL);
 }
@@ -645,7 +599,8 @@ static int l_receive (lua_State *L)
 static int l_receive_lines (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   nsock_readlines(nsp, nu->nsiod, receive_callback, nu->timeout, nu,
       luaL_checkint(L, 2));
   return yield(L, nu, "RECEIVE LINES", FROM, 0, NULL);
@@ -654,20 +609,20 @@ static int l_receive_lines (lua_State *L)
 static int l_receive_bytes (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   nsock_readbytes(nsp, nu->nsiod, receive_callback, nu->timeout, nu,
       luaL_checkint(L, 2));
   return yield(L, nu, "RECEIVE BYTES", FROM, 0, NULL);
 }
 
-#if 0
-/* Lua 5.2 */
 static int l_receive_buf (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1); /* 1 */
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   if (!(lua_type(L, 2) == LUA_TFUNCTION || lua_type(L, 2) == LUA_TSTRING))
-    luaL_typeerror(L, 2, "function/string");
+    nseU_typeerror(L, 2, "function/string");
   luaL_checktype(L, 3, LUA_TBOOLEAN); /* 3 */
 
   if (lua_getctx(L, NULL) == LUA_OK)
@@ -710,14 +665,14 @@ static int l_receive_buf (lua_State *L)
   if (lua_isnumber(L, -2) && lua_isnumber(L, -1)) /* found end? */
   {
     lua_Integer l = lua_tointeger(L, -2), r = lua_tointeger(L, -1);
-    if (l > r || r > (lua_Integer) lua_objlen(L, 5))
+    if (l > r || r > (lua_Integer) lua_rawlen(L, 5))
       return luaL_error(L, "invalid indices for match");
     lua_pushboolean(L, 1);
     if (lua_toboolean(L, 3))
       lua_pushlstring(L, lua_tostring(L, 5), r);
     else
       lua_pushlstring(L, lua_tostring(L, 5), l-1);
-    lua_pushlstring(L, lua_tostring(L, 5)+r, lua_objlen(L, 5)-r);
+    lua_pushlstring(L, lua_tostring(L, 5)+r, lua_rawlen(L, 5)-r);
     lua_rawseti(L, 4, BUFFER_I);
     return 2;
   }
@@ -728,11 +683,11 @@ static int l_receive_buf (lua_State *L)
     return yield(L, nu, "RECEIVE BUF", FROM, 0, l_receive_buf);
   }
 }
-#endif
 
 static int l_get_info (lua_State *L)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   int protocol;                                  // tcp or udp
   int af;                                        // address family
   struct sockaddr_storage local;
@@ -754,11 +709,25 @@ static int l_get_info (lua_State *L)
 
 static int l_set_timeout (lua_State *L)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
   nu->timeout = luaL_checkint(L, 2);
   if ((int) nu->timeout < -1) /* -1 is no timeout */
     return luaL_error(L, "Negative timeout: %d", nu->timeout);
-  return success(L);
+  return nseU_success(L);
+}
+
+static int sleep_destructor (lua_State *L)
+{
+  nsock_pool nsp = get_pool(L);
+  nsock_event_id *neidp = (nsock_event_id *) lua_touserdata(L, 2);
+  if (o.debugging >= 2)
+    log_write(LOG_STDERR, "Destroying sleep callback.\n");
+  assert(neidp);
+  int success = nsock_event_cancel(nsp, *neidp, 0);
+  if (success)
+    return nseU_success(L);
+  else
+    return nseU_safeerror(L, "could not cancel event");
 }
 
 static void sleep_callback (nsock_pool nsp, nsock_event nse, void *ud)
@@ -769,7 +738,7 @@ static void sleep_callback (nsock_pool nsp, nsock_event nse, void *ud)
   nse_restore(L, 0);
 }
 
-LUALIB_API int l_nsock_sleep (lua_State *L)
+static int l_sleep (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
   double secs = luaL_checknumber(L, 1);
@@ -777,9 +746,15 @@ LUALIB_API int l_nsock_sleep (lua_State *L)
 
   if (secs < 0)
     luaL_error(L, "argument to sleep (%f) must not be negative\n", secs);
+
   /* Convert to milliseconds for nsock_timer_create. */
   msecs = (int) (secs * 1000 + 0.5);
-  nsock_timer_create(nsp, sleep_callback, msecs, L);
+
+  nsock_event_id *neidp = (nsock_event_id *) lua_newuserdata(L, sizeof(nsock_event_id *));
+  *neidp = nsock_timer_create(nsp, sleep_callback, msecs, L);
+  lua_pushvalue(L, NSOCK_POOL);
+  lua_pushcclosure(L, sleep_destructor, 1);
+  nse_destructor(L, 'a');
 
   return nse_yield(L, 0, NULL);
 }
@@ -787,9 +762,9 @@ LUALIB_API int l_nsock_sleep (lua_State *L)
 #if HAVE_OPENSSL
 SSL *nse_nsock_get_ssl (lua_State *L)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
 
-  if (!nsi_checkssl(nu->nsiod))
+  if (nu->nsiod == NULL || !nsi_checkssl(nu->nsiod))
     luaL_argerror(L, 1, "not a SSL socket");
 
   return (SSL *) nsi_getssl(nu->nsiod);
@@ -808,7 +783,7 @@ int l_get_ssl_certificate (lua_State *L)
    or nil, that part of the address will be left unspecified. */
 static int l_bind (lua_State *L)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
   struct addrinfo hints = { 0 };
   struct addrinfo *results;
   const char *addr_str = luaL_optstring(L, 2, NULL);
@@ -829,12 +804,12 @@ static int l_bind (lua_State *L)
 
   rc = getaddrinfo(addr_str, port_str, &hints, &results);
   if (rc != 0)
-    return safe_error(L, gai_strerror(rc));
+    return nseU_safeerror(L, gai_strerror(rc));
   if (results == NULL)
-    return safe_error(L, "getaddrinfo: no results found");
+    return nseU_safeerror(L, "getaddrinfo: no results found");
   if (results->ai_addrlen > sizeof(nu->source_addr)) {
     freeaddrinfo(results);
-    return safe_error(L, "getaddrinfo: result is too big");
+    return nseU_safeerror(L, "getaddrinfo: result is too big");
   }
 
   /* We ignore any results after the first. */
@@ -843,7 +818,7 @@ static int l_bind (lua_State *L)
   nu->source_addrlen = results->ai_addrlen;
   memcpy(&nu->source_addr, results->ai_addr, nu->source_addrlen);
 
-  return success(L);
+  return nseU_success(L);
 }
 
 static const char *default_af_string(int af)
@@ -861,7 +836,7 @@ static void initialize (lua_State *L, int idx, nse_nsock_udata *nu,
   lua_createtable(L, 2, 0); /* room for thread in array */
   lua_pushliteral(L, "");
   lua_rawseti(L, -2, BUFFER_I);
-  lua_setfenv(L, idx);
+  lua_setuservalue(L, idx);
   nu->nsiod = NULL;
   nu->proto = proto;
   nu->af = af;
@@ -874,7 +849,7 @@ static void initialize (lua_State *L, int idx, nse_nsock_udata *nu,
   nu->direction = nu->action = NULL;
 }
 
-LUALIB_API int l_nsock_new (lua_State *L)
+static int l_new (lua_State *L)
 {
   static const char *proto_strings[] = { "tcp", "udp", NULL };
   int proto_map[] = { IPPROTO_TCP, IPPROTO_UDP };
@@ -889,32 +864,42 @@ LUALIB_API int l_nsock_new (lua_State *L)
   lua_settop(L, 0);
 
   nu = (nse_nsock_udata *) lua_newuserdata(L, sizeof(nse_nsock_udata));
-  luaL_getmetatable(L, NMAP_NSOCK_SOCKET);
+  lua_pushvalue(L, NSOCK_SOCKET);
   lua_setmetatable(L, -2);
   initialize(L, 1, nu, proto, af);
 
   return 1;
 }
 
-static int l_close (lua_State *L)
+/* Common subfunction to l_close and l_connect. l_connect calls this when a
+   second attempt is made to connect a socket that has already had a connection
+   attempt. */
+static void close_internal (lua_State *L, nse_nsock_udata *nu)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
-  if (nu->nsiod == NULL)
-    return safe_error(L, "socket already closed");
   trace(nu->nsiod, "CLOSE", TO);
 #ifdef HAVE_OPENSSL
   if (nu->ssl_session)
     SSL_SESSION_free((SSL_SESSION *) nu->ssl_session);
 #endif
-  if (!nu->is_pcap) /* pcap sockets are closed by pcap_gc */
+  if (!nu->is_pcap) { /* pcap sockets are closed by pcap_gc */
     nsi_delete(nu->nsiod, NSOCK_PENDING_NOTIFY);
+    nu->nsiod = NULL;
+  }
+}
+
+static int l_close (lua_State *L)
+{
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
+  if (nu->nsiod == NULL)
+    return nseU_safeerror(L, "socket already closed");
+  close_internal(L, nu);
   initialize(L, 1, nu, nu->proto, nu->af);
-  return success(L);
+  return nseU_success(L);
 }
 
 static int nsock_gc (lua_State *L)
 {
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
   if (nu->nsiod)
     return l_close(L);
   return 0;
@@ -954,7 +939,7 @@ static int pcap_gc (lua_State *L)
 static int l_pcap_open (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 0);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, false);
   const char *device = luaL_checkstring(L, 2);
   int snaplen = luaL_checkint(L, 3);
   luaL_checktype(L, 4, LUA_TBOOLEAN); /* promiscuous */
@@ -969,34 +954,31 @@ static int l_pcap_open (lua_State *L)
   if (nu->nsiod)
     luaL_argerror(L, 1, "socket is already open");
 
-  if (lua_objlen(L, 6) == 0)
+  if (lua_rawlen(L, 6) == 0)
     luaL_argerror(L, 2, "bad device name");
 
-  lua_rawgeti(L, LUA_ENVIRONINDEX, KEY_PCAP);
   lua_pushvalue(L, 7);
-  lua_rawget(L, -2);
+  lua_rawget(L, KEY_PCAP);
   nsock_iod *nsiod = (nsock_iod *) lua_touserdata(L, -1);
   if (nsiod == NULL) /* does not exist */
   {
+    lua_pop(L, 1); /* the nonexistant socket */
     nsiod = (nsock_iod *) lua_newuserdata(L, sizeof(nsock_iod));
-    luaL_getmetatable(L, NMAP_NSOCK_PCAP_SOCKET);
+    lua_pushvalue(L, PCAP_SOCKET);
     lua_setmetatable(L, -2);
     *nsiod = nsi_new(nsp, nu);
-    lua_rawgeti(L, LUA_ENVIRONINDEX, KEY_PCAP);
     lua_pushvalue(L, 7); /* the pcap socket key */
     lua_pushvalue(L, -2); /* the pcap socket nsiod */
-    lua_rawset(L, -3); /* _ENV[KEY_PCAP]["dev|snap|promis|bpf"] = pcap_nsiod */
-    lua_pop(L, 1); /* KEY_PCAP */
-    lua_getfenv(L, 1); /* the socket user value */
-    lua_pushvalue(L, -2); /* the pcap socket nsiod */
-    lua_pushboolean(L, 1); /* dummy variable */
-    lua_rawset(L, -3);
-    lua_pop(L, 1); /* the socket user value */
+    lua_rawset(L, KEY_PCAP); /* KEY_PCAP["dev|snap|promis|bpf"] = pcap_nsiod */
     char *e = nsock_pcap_open(nsp, *nsiod, lua_tostring(L, 6), snaplen,
         lua_toboolean(L, 4), bpf);
     if (e)
       luaL_error(L, "%s", e);
   }
+  lua_getuservalue(L, 1); /* the socket user value */
+  lua_pushvalue(L, -2); /* the pcap socket nsiod */
+  lua_pushboolean(L, 1); /* dummy variable */
+  lua_rawset(L, -3);
   nu->nsiod = *nsiod;
   nu->is_pcap = 1;
   return 0;
@@ -1030,7 +1012,8 @@ static void pcap_receive_handler (nsock_pool nsp, nsock_event nse, void *ud)
 static int l_pcap_receive (lua_State *L)
 {
   nsock_pool nsp = get_pool(L);
-  nse_nsock_udata *nu = check_nsock_udata(L, 1, 1);
+  nse_nsock_udata *nu = check_nsock_udata(L, 1, true);
+  NSOCK_UDATA_ENSURE_OPEN(L, nu);
   nu->nseid = nsock_pcap_read_packet(nsp, nu->nsiod, pcap_receive_handler,
       nu->timeout, nu);
   return yield(L, nu, "PCAP RECEIVE", FROM, 0, NULL);
@@ -1038,66 +1021,30 @@ static int l_pcap_receive (lua_State *L)
 
 LUALIB_API int luaopen_nsock (lua_State *L)
 {
-/* These two functions can be implemented in C in Lua 5.2 */
-
-  /* nsock:connect(socket, ...)
-   * This Lua function is a wrapper around the actual l_nsock_connect. The
-   * connect function must get a lock through socket_lock (C function above).
-   * Once it has the lock, it can (tail call) return the actual connect
-   * function.
-   */
-  static const char connect[] =
-"local connect, socket_lock = ...;\n"
-"return function(socket, ...)\n"
-"  repeat until socket_lock(socket) == true;\n"
-"  return connect(socket, ...);\n"
-"end\n";
-  static const char receive_buf[] =
-"local function receive_buf (socket, fstr, keep)\n"
-"  local i, j;\n"
-"  local socket_uservalue = debug.getfenv(socket);\n"
-"  local buf = socket_uservalue[2];\n"
-"  if type(fstr) == 'function' then\n"
-"    i, j = fstr(buf);\n"
-"  elseif type(fstr) == 'string' then\n"
-"    i, j = string.find(buf, fstr)\n"
-"  end\n"
-"  if type(i) == 'number' and type(j) == 'number' then\n"
-"    if i > j or j > #buf then\n"
-"      error('invalid indices for match');\n"
-"    else\n"
-"      socket_uservalue[2] = string.sub(buf, j+1);\n"
-"      if keep then\n"
-"        return true, string.sub(buf, 1, j);\n"
-"      else\n"
-"        return true, string.sub(buf, 1, i-1);\n"
-"      end\n"
-"    end\n"
-"  else\n"
-"    local status, result = socket:receive();\n"
-"    if not status then return status, result end\n"
-"    socket_uservalue[2] = socket_uservalue[2]..result;\n"
-"    return receive_buf(socket, fstr, keep);\n"
-"  end\n"
-"end\n"
-"return receive_buf;\n";
-
-  static const luaL_Reg l_nsock[] = {
+  static const luaL_Reg metatable_index[] = {
     {"bind", l_bind},
-    {"send", l_send},
-    {"sendto", l_sendto},
-    {"receive", l_receive},
-    {"receive_lines", l_receive_lines},
-    {"receive_bytes", l_receive_bytes},
-    /* {"receive_buf", l_receive_buf}, Lua 5.2 */
-    {"get_info", l_get_info},
     {"close", l_close},
-    {"set_timeout", l_set_timeout},
-    {"reconnect_ssl", l_reconnect_ssl},
+    {"connect", l_connect},
+    {"get_info", l_get_info},
     {"get_ssl_certificate", l_get_ssl_certificate},
     {"pcap_open", l_pcap_open},
     {"pcap_close", l_close},
     {"pcap_receive", l_pcap_receive},
+    {"send", l_send},
+    {"sendto", l_sendto},
+    {"receive", l_receive},
+    {"receive_buf", l_receive_buf},
+    {"receive_bytes", l_receive_bytes},
+    {"receive_lines", l_receive_lines},
+    {"reconnect_ssl", l_reconnect_ssl},
+    {"set_timeout", l_set_timeout},
+    {NULL, NULL}
+  };
+
+  static const luaL_Reg l_nsock[] = {
+    {"loop", l_loop},
+    {"new", l_new},
+    {"sleep", l_sleep},
     {NULL, NULL}
   };
 
@@ -1106,64 +1053,50 @@ LUALIB_API int luaopen_nsock (lua_State *L)
    * CONNECT_WAITING tables.
    * These values are accessed at the Lua pseudo-index LUA_ENVIRONINDEX.
    */
-  lua_createtable(L, 3, 0);
-  lua_replace(L, LUA_ENVIRONINDEX);
+  int i;
+  int top = lua_gettop(L);
 
-  weak_table(L, 0, MAX_PARALLELISM, "k");
-  lua_rawseti(L, LUA_ENVIRONINDEX, THREAD_SOCKETS);
-
-  weak_table(L, 0, 1000, "k");
-  lua_rawseti(L, LUA_ENVIRONINDEX, CONNECT_WAITING);
-
-  weak_table(L, 0, 0, "v");
-  lua_rawseti(L, LUA_ENVIRONINDEX, KEY_PCAP);
-
-  lua_pushcfunction(L, loop);
-  lua_setfield(L, LUA_REGISTRYINDEX, NSE_NSOCK_LOOP);
-
-  /* Load the connect function */
-  if (luaL_loadstring(L, connect) != 0)
-    assert(0);
-  lua_pushcfunction(L, l_connect);
-  lua_pushcfunction(L, socket_lock);
-  lua_call(L, 2, 1);                   // leave connect function on stack...
+  /* library upvalues */
+  nsock_pool nsp = new_pool(L); /* NSOCK_POOL */
+  lua_newtable(L); /* NSOCK_SOCKET */
+  lua_newtable(L); /* PCAP_SOCKET */
+  nseU_weaktable(L, 0, MAX_PARALLELISM, "k"); /* THREAD_SOCKETS */
+  nseU_weaktable(L, 0, 1000, "k"); /* CONNECT_WAITING */
+  nseU_weaktable(L, 0, 0, "v"); /* KEY_PCAP */
 
   /* Create the nsock metatable for sockets */
-  luaL_newmetatable(L, NMAP_NSOCK_SOCKET);
-  lua_createtable(L, 0, 23);
-  luaL_register(L, NULL, l_nsock);
-  lua_pushvalue(L, -3);                // connect function
-  lua_setfield(L, -2, "connect");
-  if (luaL_dostring(L, receive_buf))
-    assert(0);
-  lua_pushvalue(L, LUA_GLOBALSINDEX);
-  lua_setfenv(L, -2);
-  lua_setfield(L, -2, "receive_buf");
+  lua_pushvalue(L, top+2); /* NSOCK_SOCKET */
+  luaL_newlibtable(L, metatable_index);
+  for (i = top+1; i < top+1+6; i++) lua_pushvalue(L, i);
+  luaL_setfuncs(L, metatable_index, 6);
   lua_setfield(L, -2, "__index");
-  lua_pushcfunction(L, nsock_gc);
+  for (i = top+1; i < top+1+6; i++) lua_pushvalue(L, i);
+  lua_pushcclosure(L, nsock_gc, 6);
   lua_setfield(L, -2, "__gc");
   lua_newtable(L);
-  lua_setfield(L, -2, "__metatable");  // protect metatable
-  lua_pop(L, 1);                       // nsock metatable
+  lua_setfield(L, -2, "__metatable");  /* protect metatable */
+  lua_pop(L, 1); /* NSOCK_SOCKET */
 
   /* Create the nsock pcap metatable */
-  luaL_newmetatable(L, NMAP_NSOCK_PCAP_SOCKET);
-  lua_pushcfunction(L, pcap_gc);
-  lua_setfield(L, -2, "__gc");
-  lua_pop(L, 1);
+  lua_pushvalue(L, top+3); /* PCAP_SOCKET */
+  for (i = top+1; i < top+1+6; i++) lua_pushvalue(L, i);
+  lua_pushcclosure(L, pcap_gc, 6);
+  lua_setfield(L, top+3, "__gc");
+  lua_pop(L, 1); /* PCAP_SOCKET */
 
 #if HAVE_OPENSSL
   /* Set up the SSL certificate userdata code in nse_ssl_cert.cc. */
   nse_nsock_init_ssl_cert(L);
 #endif
 
-  nsock_pool nsp = new_pool(L);
-  if (o.scriptTrace())
-    nsp_settrace(nsp, NULL, NSOCK_TRACE_LEVEL, o.getStartTime());
 #if HAVE_OPENSSL
   /* Value speed over security in SSL connections. */
   nsp_ssl_init_max_speed(nsp);
 #endif
 
-  return 0;
+  luaL_newlibtable(L, l_nsock);
+  for (i = top+1; i < top+1+6; i++) lua_pushvalue(L, i);
+  luaL_setfuncs(L, l_nsock, 6);
+
+  return 1;
 }
