@@ -6,12 +6,16 @@
 -- 
 -- @copyright Same as Nmap--See http://nmap.org/book/man-legal.html
 
-require "stdnse"
+local coroutine = require "coroutine"
+local debug = require "debug"
+local io = require "io"
+local math = require "math"
+local stdnse = require "stdnse"
+local string = require "string"
+_ENV = stdnse.module("nsedebug", stdnse.seeall)
 
 local EMPTY = {}; -- Empty constant table
 
-module(... or "nsedebug", package.seeall);
-
 ---
 -- Converts an arbitrary data type into a string. Will recursively convert 
 -- tables. This can be very useful for debugging. 
@@ -58,11 +62,13 @@ function tostr(data, indent)
 	return str
 end
 
--- Print out a string in hex, for debugging. 
+--- Print out a string in hex, for debugging.
+--
+--@param str The data to print in hex.
 function print_hex(str)
 
 	-- Prints out the full lines
-	for line=1, string.len(str)/16, 1 do
+	for line=1, #str/16, 1 do
 		io.write(string.format("%08x ", (line - 1) * 16))
 
 		-- Loop through the string, printing the hex
@@ -86,26 +92,30 @@ function print_hex(str)
 	end
 
 	-- Prints out the final, partial line
-	local line = math.floor((string.len(str)/16)) + 1
-	io.write(string.format("%08x ", (line - 1) * 16))
+	if (#str % 16 ~= 0) then
+		local line = math.floor((#str/16)) + 1
+		io.write(string.format("%08x ", (line - 1) * 16))
 
-	for char=1, string.len(str) % 16, 1 do
-		local ch = string.byte(str, ((line - 1) * 16) + char)
-		io.write(string.format("%02x ", ch))
-	end
-	io.write(string.rep("   ", 16 - (string.len(str) % 16)));
-	io.write("   ")
-
-	for char=1, string.len(str) % 16, 1 do
-		local ch = string.byte(str, ((line - 1) * 16) + char)
-		if ch < 0x20 or ch > 0x7f then
-			ch = string.byte(".", 1)
+		for char=1, #str % 16, 1 do
+			local ch = string.byte(str, ((line - 1) * 16) + char)
+			io.write(string.format("%02x ", ch))
 		end
-		io.write(string.format("%c", ch))
+		io.write(string.rep("   ", 16 - (#str % 16)));
+		io.write("   ")
+
+		for char=1, #str % 16, 1 do
+			local ch = string.byte(str, ((line - 1) * 16) + char)
+			if ch < 0x20 or ch > 0x7f then
+				ch = string.byte(".", 1)
+			end
+			io.write(string.format("%c", ch))
+		end
+		
+		io.write("\n")
 	end
 
 	-- Print out the length
-	io.write(string.format("\n         Length: %d [0x%x]\n", string.len(str), string.len(str)))
+	io.write(string.format("         Length: %d [0x%x]\n", #str, #str))
 
 end
 
@@ -119,3 +129,5 @@ function print_stack()
 end
 
 
+
+return _ENV;
