@@ -1,4 +1,21 @@
---- URI parsing, composition, and relative URL resolution.
+---
+-- URI parsing, composition, and relative URL resolution.
+--
+-- A URL is represented as a table with the following entries:
+-- * <code>scheme</code>
+-- * <code>fragment</code>
+-- * <code>query</code>
+-- * <code>params</code>
+-- * <code>authority</code>
+-- * <code>userinfo</code>
+-- * <code>path</code>
+-- * <code>port</code>
+-- * <code>password</code>
+-- These correspond to these parts of a URL (some may be <code>nil</code>):
+-- <code>
+-- scheme://userinfo@password:authority:port/path;params?query#fragment
+-- </code>
+--
 -- @author Diego Nehab
 -- @author Eddie Bell <ejlbell@gmail.com>
 
@@ -14,10 +31,15 @@ parse_query and build_query added For nmap (Eddie Bell <ejlbell@gmail.com>)
 -----------------------------------------------------------------------------
 -- Declare module
 -----------------------------------------------------------------------------
-local string = require("string")
+
+local _G = require "_G"
+local stdnse = require "stdnse"
+local string = require "string"
+local table = require "table"
 local base = _G
-local table = require("table")
-module(... or "url",package.seeall)
+
+
+_ENV = stdnse.module("url", stdnse.seeall)
 
 _VERSION = "URL 1.0"
 
@@ -130,8 +152,6 @@ function parse(url, default)
     -- initialize default parameters
     local parsed = {}
     for i,v in base.pairs(default or parsed) do parsed[i] = v end
-    -- empty url is parsed to nil
-    if not url or url == "" then return nil, "invalid url" end
     -- remove whitespace
     -- url = string.gsub(url, "%s", "")
     -- get fragment
@@ -139,9 +159,9 @@ function parse(url, default)
         parsed.fragment = f
         return ""
     end)
-    -- get scheme
+    -- get scheme. Lower-case according to RFC 3986 section 3.1.
     url = string.gsub(url, "^([%w][%w%+%-%.]*)%:",
-        function(s) parsed.scheme = s; return "" end)
+        function(s) parsed.scheme = string.lower(s); return "" end)
     -- get authority
     url = string.gsub(url, "^//([^/]*)", function(n)
         parsed.authority = n
@@ -158,13 +178,13 @@ function parse(url, default)
         return ""
     end)
     -- path is whatever was left
-    if url ~= "" then parsed.path = url end
+    parsed.path = url
     local authority = parsed.authority
     if not authority then return parsed end
     authority = string.gsub(authority,"^([^@]*)@",
         function(u) parsed.userinfo = u; return "" end)
-    authority = string.gsub(authority, ":([^:]*)$",
-        function(p) parsed.port = p; return "" end)
+    authority = string.gsub(authority, ":([0-9]*)$",
+        function(p) if p ~= "" then parsed.port = p end; return "" end)
     if authority ~= "" then parsed.host = authority end
     local userinfo = parsed.userinfo
     if not userinfo then return parsed end
@@ -213,6 +233,7 @@ end
 -- @return The corresponding absolute URL.
 -----------------------------------------------------------------------------
 function absolute(base_url, relative_url)
+    local base_parsed;
     if type(base_url) == "table" then
         base_parsed = base_url
         base_url = build(base_parsed)
@@ -254,8 +275,8 @@ function parse_path(path)
 	path = path or ""
 	--path = string.gsub(path, "%s", "")
 	string.gsub(path, "([^/]+)", function (s) table.insert(parsed, s) end)
-	for i = 1, table.getn(parsed) do
-		parsed[i] = unescape(parsed[i])
+	for i, v in ipairs(parsed) do
+		parsed[i] = unescape(v)
 	end
 	if string.sub(path, 1, 1) == "/" then parsed.is_absolute = 1 end
 	if string.sub(path, -1, -1) == "/" then parsed.is_directory = 1 end
@@ -270,7 +291,7 @@ end
 -----------------------------------------------------------------------------
 function build_path(parsed, unsafe)
 	local path = ""
-	local n = table.getn(parsed)
+	local n = #parsed
 	if unsafe then
 		for i = 1, n-1 do
 			path = path .. parsed[i]
@@ -313,7 +334,7 @@ function parse_query(query)
 	query = string.gsub(query, "&lt;", "<")
 	query = string.gsub(query, "&gt;", ">")
 
-	function ginsert(qstr)
+	local function ginsert(qstr)
 		local first, last = string.find(qstr, "=")
 		if first then
 			parsed[string.sub(qstr, 0, first-1)] = string.sub(qstr, first+1)
@@ -347,5 +368,7 @@ function build_query(query)
 	for i,v in pairs(query) do 
 		qstr = qstr .. i .. '=' .. v .. '&'
 	end
-	return string.sub(qstr, 0, string.len(qstr)-1)
+	return string.sub(qstr, 0, #qstr-1)
 end	
+
+return _ENV;
