@@ -6,59 +6,91 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
+ *                                                                         *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
  * continued development of Nmap.  Please email sales@insecure.com for     *
  * further information.                                                    *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
@@ -70,8 +102,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -85,14 +117,13 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: targets.cc 28528 2012-05-04 15:12:25Z david $ */
+/* $Id: targets.cc 31563 2013-07-28 22:08:48Z fyodor $ */
 
 
 #include "nbase/nbase_addrset.h"
@@ -106,15 +137,15 @@
 #include "nmap_tty.h"
 #include "utils.h"
 #include "xml.h"
-using namespace std;
+
 extern NmapOps o;
 
 /* Conducts an ARP ping sweep of the given hosts to determine which ones
    are up on a local ethernet network */
 static void arpping(Target *hostbatch[], int num_hosts) {
-  /* First I change hostbatch into a vector<Target *>, which is what ultra_scan
+  /* First I change hostbatch into a std::vector<Target *>, which is what ultra_scan
      takes.  I remove hosts that cannot be ARP scanned (such as localhost) */
-  vector<Target *> targets;
+  std::vector<Target *> targets;
   int targetno;
   targets.reserve(num_hosts);
 
@@ -303,13 +334,163 @@ static bool target_needs_new_hostgroup(const HostGroupState *hs, const Target *t
      replies. What happens is one target gets the replies for all probes
      referring to the same IP address. */
   for (i = 0; i < hs->current_batch_sz; i++) {
-    if (sockaddr_storage_cmp(hs->hostbatch[0]->TargetSockAddr(), target->TargetSockAddr()) == 0)
+    if (sockaddr_storage_cmp(hs->hostbatch[i]->TargetSockAddr(), target->TargetSockAddr()) == 0)
       return true;
   }
 
   return false;
 }
 
+/* Initializes (or reinitializes) the object with a new expression, such
+   as 192.168.0.0/16 , 10.1.0-5.1-254 , or fe80::202:e3ff:fe14:1102 .
+   Returns 0 for success */
+int TargetGroup::parse_expr(const char *target_expr, int af) {
+  if (this->netblock != NULL)
+    delete this->netblock;
+  this->netblock = NetBlock::parse_expr(target_expr, af);
+  if (this->netblock != NULL)
+    return 0;
+  else
+    return 1;
+}
+
+/* Grab the next host from this expression (if any) and updates its internal
+   state to reflect that the IP was given out.  Returns 0 and
+   fills in ss if successful.  ss must point to a pre-allocated
+   sockaddr_storage structure */
+int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
+  if (this->netblock == NULL)
+    return -1;
+
+  /* If all we have at this point is a hostname and netmask, resolve into
+     something where we know the address. If we ever have to use strictly the
+     hostname, without doing local DNS resolution (like with a proxy scan), this
+     has to be made conditional (and perhaps an error if the netmask doesn't
+     limit it to exactly one address). */
+  NetBlockHostname *netblock_hostname;
+  netblock_hostname = dynamic_cast<NetBlockHostname *>(this->netblock);
+  if (netblock_hostname != NULL) {
+    this->netblock = netblock_hostname->resolve();
+    if (this->netblock == NULL) {
+      error("Failed to resolve \"%s\".", netblock_hostname->hostname.c_str());
+      return -1;
+    }
+    delete netblock_hostname;
+  }
+
+  /* Check for proper address family. Give a specific error message for IPv6
+     specifications appearing in IPv4 mode. */
+  if (o.af() == AF_INET && dynamic_cast<NetBlockIPv6Netmask *>(this->netblock) != NULL) {
+    error("%s looks like an IPv6 target specification -- you have to use the -6 option.",
+      this->netblock->str().c_str());
+    return -1;
+  }
+  if ((o.af() == AF_INET && dynamic_cast<NetBlockIPv4Ranges *>(this->netblock) == NULL) ||
+      (o.af() == AF_INET6 && dynamic_cast<NetBlockIPv6Netmask *>(this->netblock) == NULL)) {
+    error("Address family mismatch in target specification \"%s\".",
+      this->netblock->str().c_str());
+    return -1;
+  }
+
+  if (this->netblock->next(ss, sslen))
+    return 0;
+  else
+    return -1;
+}
+
+/* Returns true iff the given address is the one that was resolved to create
+   this target group; i.e., not one of the addresses derived from it with a
+   netmask. */
+bool TargetGroup::is_resolved_address(const struct sockaddr_storage *ss) const {
+  return this->netblock->is_resolved_address(ss);
+}
+
+/* Return a string of the name or address that was resolved for this group. */
+const char *TargetGroup::get_resolved_name(void) const {
+  if (this->netblock->hostname.empty())
+    return NULL;
+  else
+    return this->netblock->hostname.c_str();
+}
+
+/* Return the list of addresses that the name for this group resolved to, if
+   it came from a name resolution. */
+const std::list<struct sockaddr_storage> &TargetGroup::get_resolved_addrs(void) const {
+  return this->netblock->resolvedaddrs;
+}
+
+/* is the current expression a named host */
+int TargetGroup::get_namedhost() const {
+  return this->get_resolved_name() != NULL;
+}
+
+/* Lookahead is the number of hosts that can be
+   checked (such as ping scanned) in advance.  Randomize causes each
+   group of up to lookahead hosts to be internally shuffled around.
+   The target_expressions array MUST REMAIN VALID IN MEMORY as long as
+   this class instance is used -- the array is NOT copied.
+ */
+HostGroupState::HostGroupState(int lookahead, int rnd, int argc, const char **argv) {
+  assert(lookahead > 0);
+  this->argc = argc;
+  this->argv = argv;
+  hostbatch = (Target **) safe_zalloc(sizeof(Target *) * lookahead);
+  defer_buffer = std::list<Target *>();
+  undeferred = std::list<Target *>();
+  max_batch_sz = lookahead;
+  current_batch_sz = 0;
+  next_batch_no = 0;
+  randomize = rnd;
+}
+
+HostGroupState::~HostGroupState() {
+  free(hostbatch);
+}
+
+/* Returns true iff the defer buffer is not yet full. */
+bool HostGroupState::defer(Target *t) {
+  this->defer_buffer.push_back(t);
+  return this->defer_buffer.size() < HostGroupState::DEFER_LIMIT;
+}
+
+void HostGroupState::undefer() {
+  this->undeferred.splice(this->undeferred.end(), this->defer_buffer);
+}
+
+const char *HostGroupState::next_expression() {
+  static char buf[1024];
+
+  if (o.max_ips_to_scan == 0 || o.numhosts_scanned + this->current_batch_sz < o.max_ips_to_scan) {
+    const char *expr;
+    expr = grab_next_host_spec(o.inputfd, o.generate_random_ips, this->argc, this->argv);
+    if (expr != NULL)
+      return expr;
+  }
+
+#ifndef NOLUA
+  /* Add any new NSE discovered targets to the scan queue */
+
+  NewTargets *new_targets = NewTargets::get();
+  if (o.script && new_targets != NULL) {
+    if (new_targets->get_queued() > 0) {
+      std::string expr_string;
+      expr_string = new_targets->read().c_str();
+      if (o.debugging > 3) {
+        log_write(LOG_PLAIN,
+                  "New targets in the scanned cache: %ld, pending ones: %ld.\n",
+                  new_targets->get_scanned(), new_targets->get_queued());
+      }
+      if (!expr_string.empty()) {
+        Strncpy(buf, expr_string.c_str(), sizeof(buf));
+        return buf;
+      }
+    }
+  }
+#endif
+
+  return NULL;
+}
+
 /* Add a <target> element to the XML stating that a target specification was
    ignored. This can be because of, for example, a DNS resolution failure, or a
    syntax error. */
@@ -322,109 +503,151 @@ static void log_bogus_target(const char *expr) {
   xml_newline();
 }
 
-Target *nexthost(HostGroupState *hs, const addrset *exclude_group,
-                 struct scan_lists *ports, int pingtype) {
-  int i;
+/* Returns a newly allocated Target with the given address. Handles all the
+   details like setting the Target's address and next hop. */
+static Target *setup_target(const HostGroupState *hs,
+                            const struct sockaddr_storage *ss, size_t sslen,
+                            int pingtype) {
+  struct route_nfo rnfo;
+  Target *t;
+
+  t = new Target();
+
+  t->setTargetSockAddr(ss, sslen);
+
+  /* Special handling for the resolved address (for example whatever
+     scanme.nmap.org resolves to in scanme.nmap.org/24). */
+  if (hs->current_group.is_resolved_address(ss)) {
+    if (hs->current_group.get_namedhost())
+      t->setTargetName(hs->current_group.get_resolved_name());
+    t->resolved_addrs = hs->current_group.get_resolved_addrs();
+  }
+
+  /* We figure out the source IP/device IFF
+     1) We are r00t AND
+     2) We are doing tcp or udp pingscan OR
+     3) We are doing a raw-mode portscan or osscan or traceroute OR
+     4) We are on windows and doing ICMP ping */
+  if (o.isr00t &&
+      ((pingtype & (PINGTYPE_TCP|PINGTYPE_UDP|PINGTYPE_SCTP_INIT|PINGTYPE_PROTO|PINGTYPE_ARP)) || o.RawScan()
+#ifdef WIN32
+       || (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))
+#endif // WIN32
+      )) {
+    if (!nmap_route_dst(ss, &rnfo)) {
+      log_bogus_target(inet_ntop_ez(ss, sslen));
+      error("%s: failed to determine route to %s", __func__, t->NameIP());
+      goto bail;
+    }
+    if (rnfo.direct_connect) {
+      t->setDirectlyConnected(true);
+    } else {
+      t->setDirectlyConnected(false);
+      t->setNextHop(&rnfo.nexthop, sizeof(rnfo.nexthop));
+    }
+    t->setIfType(rnfo.ii.device_type);
+    if (rnfo.ii.device_type == devt_ethernet) {
+      if (o.spoofMACAddress())
+        t->setSrcMACAddress(o.spoofMACAddress());
+      else
+        t->setSrcMACAddress(rnfo.ii.mac);
+    }
+    t->setSourceSockAddr(&rnfo.srcaddr, sizeof(rnfo.srcaddr));
+    if (hs->current_batch_sz == 0) /* Because later ones can have different src addy and be cut off group */
+      o.decoys[o.decoyturn] = t->v4source();
+    t->setDeviceNames(rnfo.ii.devname, rnfo.ii.devfullname);
+    t->setMTU(rnfo.ii.mtu);
+    // printf("Target %s %s directly connected, goes through local iface %s, which %s ethernet\n", t->NameIP(), t->directlyConnected()? "IS" : "IS NOT", t->deviceName(), (t->ifType() == devt_ethernet)? "IS" : "IS NOT");
+  }
+
+  return t;
+
+bail:
+  delete t;
+  return NULL;
+}
+
+static Target *next_target(HostGroupState *hs, const addrset *exclude_group,
+  struct scan_lists *ports, int pingtype) {
   struct sockaddr_storage ss;
   size_t sslen;
-  struct route_nfo rnfo;
+  Target *t;
+
+  /* First handle targets deferred in the last batch. */
+  if (!hs->undeferred.empty()) {
+    t = hs->undeferred.front();
+    hs->undeferred.pop_front();
+    return t;
+  }
+
+tryagain:
+
+  if (hs->current_group.get_next_host(&ss, &sslen) != 0) {
+    const char *expr;
+    /* We are going to have to pop in another expression. */
+    for (;;) {
+      expr = hs->next_expression();
+      if (expr == NULL)
+        /* That's the last of them. */
+        return NULL;
+      if (hs->current_group.parse_expr(expr, o.af()) == 0)
+        break;
+      else
+        log_bogus_target(expr);
+    }
+    goto tryagain;
+  }
+
+  assert(ss.ss_family == o.af());
+
+  /* If we are resuming from a previous scan, we have already finished scanning
+     up to o.resume_ip.  */
+  if (ss.ss_family == AF_INET && o.resume_ip.s_addr) {
+    if (o.resume_ip.s_addr == ((struct sockaddr_in *) &ss)->sin_addr.s_addr)
+      /* We will continue starting with the next IP. */
+      o.resume_ip.s_addr = 0;
+    goto tryagain;
+  }
+
+  /* Check exclude list. */
+  if (hostInExclude((struct sockaddr *) &ss, sslen, exclude_group))
+    goto tryagain;
+
+  t = setup_target(hs, &ss, sslen, pingtype);
+  if (t == NULL)
+    goto tryagain;
+
+  return t;
+}
+
+static void refresh_hostbatch(HostGroupState *hs, const addrset *exclude_group,
+  struct scan_lists *ports, int pingtype) {
+  int i;
   bool arpping_done = false;
   struct timeval now;
 
-  if (hs->next_batch_no < hs->current_batch_sz) {
-    /* Woop!  This is easy -- we just pass back the next host struct */
-    return hs->hostbatch[hs->next_batch_no++];
-  }
-  /* Doh, we need to refresh our array */
-  /* for (i=0; i < hs->max_batch_sz; i++) hs->hostbatch[i] = new Target(); */
-
   hs->current_batch_sz = hs->next_batch_no = 0;
-  do {
-    /* Grab anything we have in our current_expression */
-    while (hs->current_batch_sz < hs->max_batch_sz && 
-        hs->current_expression.get_next_host(&ss, &sslen) == 0) {
-      Target *t;
+  hs->undefer();
+  while (hs->current_batch_sz < hs->max_batch_sz) {
+    Target *t;
 
-      if (hostInExclude((struct sockaddr *)&ss, sslen, exclude_group)) {
-        continue; /* Skip any hosts the user asked to exclude */
-      }
-      t = new Target();
-      t->setTargetSockAddr(&ss, sslen);
+    t = next_target(hs, exclude_group, ports, pingtype);
+    if (t == NULL)
+      break;
 
-      /* Special handling for the resolved address (for example whatever
-         scanme.nmap.org resolves to in scanme.nmap.org/24). */
-      if (hs->current_expression.is_resolved_address(&ss)) {
-        if (hs->current_expression.get_namedhost())
-          t->setTargetName(hs->current_expression.get_resolved_name());
-        t->resolved_addrs = hs->current_expression.get_resolved_addrs();
-      }
-
-      /* We figure out the source IP/device IFF
-         1) We are r00t AND
-         2) We are doing tcp or udp pingscan OR
-         3) We are doing a raw-mode portscan or osscan or traceroute OR
-         4) We are on windows and doing ICMP ping */
-      if (o.isr00t && 
-          ((pingtype & (PINGTYPE_TCP|PINGTYPE_UDP|PINGTYPE_SCTP_INIT|PINGTYPE_PROTO|PINGTYPE_ARP)) || o.RawScan()
-#ifdef WIN32
-           || (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))
-#endif // WIN32
-          )) {
-        t->TargetSockAddr(&ss, &sslen);
-        if (!nmap_route_dst(&ss, &rnfo)) {
-          fatal("%s: failed to determine route to %s", __func__, t->NameIP());
-        }
-        if (rnfo.direct_connect) {
-          t->setDirectlyConnected(true);
-        } else {
-          t->setDirectlyConnected(false);
-          t->setNextHop(&rnfo.nexthop, sizeof(rnfo.nexthop));
-        }
-        t->setIfType(rnfo.ii.device_type);
-        if (rnfo.ii.device_type == devt_ethernet) {
-          if (o.spoofMACAddress())
-            t->setSrcMACAddress(o.spoofMACAddress());
-          else
-            t->setSrcMACAddress(rnfo.ii.mac);
-        }
-        t->setSourceSockAddr(&rnfo.srcaddr, sizeof(rnfo.srcaddr));
-        if (hs->current_batch_sz == 0) /* Because later ones can have different src addy and be cut off group */
-          o.decoys[o.decoyturn] = t->v4source();
-        t->setDeviceNames(rnfo.ii.devname, rnfo.ii.devfullname);
-        t->setMTU(rnfo.ii.mtu);
-        // printf("Target %s %s directly connected, goes through local iface %s, which %s ethernet\n", t->NameIP(), t->directlyConnected()? "IS" : "IS NOT", t->deviceName(), (t->ifType() == devt_ethernet)? "IS" : "IS NOT");
-      }
-
-      /* Does this target need to go in a separate host group? */
-      if (target_needs_new_hostgroup(hs, t)) {
-        /* Cancel everything!  This guy must go in the next group and we are
-           out of here */
-        hs->current_expression.return_last_host();
-        delete t;
-        goto batchfull;
-      }
-
-      hs->hostbatch[hs->current_batch_sz++] = t;
+    /* Does this target need to go in a separate host group? */
+    if (target_needs_new_hostgroup(hs, t)) {
+      if (hs->defer(t))
+        continue;
+      else
+        break;
     }
 
-    if (hs->current_batch_sz < hs->max_batch_sz &&
-        hs->next_expression < hs->num_expressions) {
-      /* We are going to have to pop in another expression. */
-      while (hs->next_expression < hs->num_expressions) {
-        const char *expr;
-        expr = hs->target_expressions[hs->next_expression++];
-        if (hs->current_expression.parse_expr(expr, o.af()) != 0)
-          log_bogus_target(expr);
-        else
-          break;
-      }
-    } else break;
-  } while(1);
-
-batchfull:
+    hs->hostbatch[hs->current_batch_sz++] = t;
+  }
 
   if (hs->current_batch_sz == 0)
-    return NULL;
+    return;
 
   /* OK, now we have our complete batch of entries.  The next step is to
      randomize them (if requested) */
@@ -438,7 +661,8 @@ batchfull:
   if (hs->hostbatch[0]->ifType() == devt_ethernet && 
       hs->hostbatch[0]->af() == AF_INET &&
       hs->hostbatch[0]->directlyConnected() && 
-      o.sendpref != PACKET_SEND_IP_STRONG) {
+      o.sendpref != PACKET_SEND_IP_STRONG &&
+      (pingtype == PINGTYPE_ARP || o.implicitARPPing)) {
     arpping(hs->hostbatch, hs->current_batch_sz);
     arpping_done = true;
   }
@@ -448,7 +672,8 @@ batchfull:
   if (hs->hostbatch[0]->ifType() == devt_ethernet &&
       hs->hostbatch[0]->af() == AF_INET6 &&
       hs->hostbatch[0]->directlyConnected() &&
-      o.sendpref != PACKET_SEND_IP_STRONG) {
+      o.sendpref != PACKET_SEND_IP_STRONG &&
+      (pingtype == PINGTYPE_ARP || o.implicitARPPing)) {
     arpping(hs->hostbatch, hs->current_batch_sz);
     arpping_done = true;
   }
@@ -467,8 +692,6 @@ batchfull:
     }
   }
 
-  /* TODO: Maybe I should allow real ping scan of directly connected
-     ethernet hosts? */
   /* Then we do the mass ping (if required - IP-level pings) */
   if ((pingtype == PINGTYPE_NONE && !arpping_done) || hs->hostbatch[0]->ifType() == devt_loopback) {
     for (i=0; i < hs->current_batch_sz; i++) {
@@ -487,6 +710,14 @@ batchfull:
 
   if (!o.noresolve)
     nmap_mass_rdns(hs->hostbatch, hs->current_batch_sz);
+}
+
+Target *nexthost(HostGroupState *hs, const addrset *exclude_group,
+                 struct scan_lists *ports, int pingtype) {
+  if (hs->next_batch_no >= hs->current_batch_sz)
+    refresh_hostbatch(hs, exclude_group, ports, pingtype);
+  if (hs->next_batch_no >= hs->current_batch_sz)
+    return NULL;
 
   return hs->hostbatch[hs->next_batch_no++];
 }
