@@ -3,59 +3,91 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
+ *                                                                         *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two.                                  *
+ *                                                                         *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the special and conditions of the license text as well.       *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
  * continued development of Nmap.  Please email sales@insecure.com for     *
  * further information.                                                    *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
- *                                                                         *
  * If you received these files with a written license agreement or         *
  * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
@@ -67,8 +99,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -82,10 +114,9 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
@@ -166,23 +197,6 @@ struct Hop;
 class HostState;
 class Probe;
 
-/* A global random token used to distinguish this traceroute's probes from
-   those of other traceroutes possibly running on the same machine. */
-static u16 global_id;
-/* A global cache of known hops, indexed by TTL and address. */
-static std::map<struct HopIdent, Hop *> hop_cache;
-/* A list of timedout hops, which are not kept in hop_cache, so we can delete
-   all hops on occasion. */
-static std::list<Hop *> timedout_hops;
-/* The TTL at which we start sending probes if we don't have a distance
-   estimate. This is updated after each host group on the assumption that hosts
-   across groups will not differ much in distance. Having this closer to the
-   true distance makes the trace faster but is not needed for accuracy. */
-static u8 initial_ttl = 10;
-
-static struct timeval get_now(struct timeval *now = NULL);
-static const char *ss_to_string(const struct sockaddr_storage *ss);
-
 /* An object of this class is a (TTL, address) pair that uniquely identifies a
    hop. Hops in the hop_cache are indexed by this type. */
 struct HopIdent {
@@ -204,6 +218,23 @@ struct HopIdent {
   }
 };
 
+/* A global random token used to distinguish this traceroute's probes from
+   those of other traceroutes possibly running on the same machine. */
+static u16 global_id;
+/* A global cache of known hops, indexed by TTL and address. */
+static std::map<struct HopIdent, Hop *> hop_cache;
+/* A list of timedout hops, which are not kept in hop_cache, so we can delete
+   all hops on occasion. */
+static std::list<Hop *> timedout_hops;
+/* The TTL at which we start sending probes if we don't have a distance
+   estimate. This is updated after each host group on the assumption that hosts
+   across groups will not differ much in distance. Having this closer to the
+   true distance makes the trace faster but is not needed for accuracy. */
+static u8 initial_ttl = 10;
+
+static struct timeval get_now(struct timeval *now = NULL);
+static const char *ss_to_string(const struct sockaddr_storage *ss);
+
 struct Hop {
   Hop *parent;
   struct sockaddr_storage tag;
@@ -273,7 +304,7 @@ private:
   /* This is incremented with each instantiated probe. */
   static u16 token_counter;
 
-  int num_resends;
+  unsigned int num_resends;
 
 public:
   HostState *host;
@@ -555,8 +586,11 @@ struct probespec HostState::get_probe(const Target *target) {
   struct probespec probe;
 
   probe = target->pingprobe;
-  if (probe.type == PS_TCP || probe.type == PS_UDP || probe.type == PS_ICMP ||
-      probe.type == PS_SCTP || probe.type == PS_ICMPV6) {
+  if (target->af() == AF_INET &&
+      (probe.type == PS_TCP || probe.type == PS_UDP || probe.type == PS_SCTP || probe.type == PS_ICMP)) {
+    /* Nothing needed. */
+  } else if (target->af() == AF_INET6 &&
+      (probe.type == PS_TCP || probe.type == PS_UDP || probe.type == PS_SCTP || probe.type == PS_ICMPV6)) {
     /* Nothing needed. */
   } else if (probe.type == PS_PROTO) {
     /* If this is an IP protocol probe, fill in some fields for some common
@@ -571,6 +605,10 @@ struct probespec HostState::get_probe(const Target *target) {
       probe.pd.sctp.dport = get_random_u16();
     } else if (probe.proto == IPPROTO_ICMP) {
       probe.pd.icmp.type = ICMP_ECHO;
+    } else if (probe.proto == IPPROTO_ICMPV6) {
+      probe.pd.icmp.type = ICMPV6_ECHO;
+    } else {
+      fatal("Unknown protocol %d", probe.proto);
     }
   } else {
     /* No responsive probe known? The user probably skipped both ping and
@@ -729,9 +767,6 @@ public:
     } else {
       fatal("Unknown address family %u in %s.", source->ss_family, __func__);
     }
-
-    /* This should not be reached. Just in case. */
-    assert(0);
   }
 };
 
@@ -744,18 +779,25 @@ public:
   unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
     const char *payload;
     size_t payload_length;
-    const struct sockaddr_in *sin;
-
-    assert(source->ss_family == AF_INET);
-    sin = (struct sockaddr_in *) source;
 
     payload = get_udp_payload(pspec.pd.udp.dport, &payload_length);
 
     /* For UDP we encode the token in the source port. */
-    return build_udp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
-      get_random_u16(), get_random_u8(), false, NULL, 0,
-      token ^ global_id, pspec.pd.udp.dport,
-      payload, payload_length, len);
+    if (source->ss_family == AF_INET) {
+      const struct sockaddr_in *sin = (struct sockaddr_in *) source;
+      return build_udp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
+        get_random_u16(), get_random_u8(), false, NULL, 0,
+        token ^ global_id, pspec.pd.udp.dport,
+        payload, payload_length, len);
+    } else if (source->ss_family == AF_INET6) {
+      const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) source;
+      return build_udp_raw_ipv6(&sin6->sin6_addr, host->target->v6hostip(),
+        0, 0, ttl,
+        token ^ global_id, pspec.pd.udp.dport,
+        payload, payload_length, len);
+    } else {
+      fatal("Unknown address family %u in %s.", source->ss_family, __func__);
+    }
   }
 };
 
@@ -767,18 +809,27 @@ public:
   }
   unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
     struct sctp_chunkhdr_init chunk;
-    const struct sockaddr_in *sin;
-
-    assert(source->ss_family == AF_INET);
-    sin = (struct sockaddr_in *) source;
 
     sctp_pack_chunkhdr_init(&chunk, SCTP_INIT, 0, sizeof(chunk),
       get_random_u32() /*itag*/, 32768, 10, 2048, get_random_u32() /*itsn*/);
-    return build_sctp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
-      get_random_u16(), get_random_u8(), false, NULL, 0,
-      token ^ global_id, pspec.pd.sctp.dport, 0UL,
-      (char *) &chunk, sizeof(chunk),
-      o.extra_payload, o.extra_payload_length, len);
+
+    if (source->ss_family == AF_INET) {
+      const struct sockaddr_in *sin = (struct sockaddr_in *) source;
+      return build_sctp_raw(&sin->sin_addr, host->target->v4hostip(), ttl,
+        get_random_u16(), get_random_u8(), false, NULL, 0,
+        token ^ global_id, pspec.pd.sctp.dport, 0UL,
+        (char *) &chunk, sizeof(chunk),
+        o.extra_payload, o.extra_payload_length, len);
+    } else if (source->ss_family == AF_INET6) {
+      const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) source;
+      return build_sctp_raw_ipv6(&sin6->sin6_addr, host->target->v6hostip(),
+        0, 0, ttl,
+        token ^ global_id, pspec.pd.sctp.dport, 0UL,
+        (char *) &chunk, sizeof(chunk),
+        o.extra_payload, o.extra_payload_length, len);
+    } else {
+      fatal("Unknown address family %u in %s.", source->ss_family, __func__);
+    }
   }
 };
 
@@ -789,13 +840,20 @@ public:
   : Probe(host, pspec, ttl) {
   }
   unsigned char *build_packet(const struct sockaddr_storage *source, u32 *len) const {
-    const struct sockaddr_in *sin;
-    assert(source->ss_family == AF_INET);
-    sin = (struct sockaddr_in *) source;
-    /* For IP proto scan the token is put in the IP ID. */
-    return build_ip_raw(&sin->sin_addr, host->target->v4hostip(), pspec.proto, ttl,
-      token ^ global_id, get_random_u8(), false, NULL, 0,
-      o.extra_payload, o.extra_payload_length, len);
+    /* For IP proto scan the token is put in the IP ID or flow label. */
+    if (source->ss_family == AF_INET) {
+      const struct sockaddr_in *sin = (struct sockaddr_in *) source;
+      return build_ip_raw(&sin->sin_addr, host->target->v4hostip(), pspec.proto, ttl,
+        token ^ global_id, get_random_u8(), false, NULL, 0,
+        o.extra_payload, o.extra_payload_length, len);
+    } else if (source->ss_family == AF_INET6) {
+      const struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) source;
+      return build_ipv6_raw(&sin6->sin6_addr, host->target->v6hostip(),
+        0, token ^ global_id, pspec.proto, ttl,
+        o.extra_payload, o.extra_payload_length, len);
+    } else {
+      fatal("Unknown address family %u in %s.", source->ss_family, __func__);
+    }
   }
 };
 
@@ -854,13 +912,9 @@ TracerouteState::TracerouteState(std::vector<Target *> &targets) {
 #ifdef WIN32
     win32_fatal_raw_sockets(targets[0]->deviceName());
 #endif
-    rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
-    if (rawsd == -1)
+    rawsd = nmap_raw_socket();
+    if (rawsd < 0)
       pfatal("traceroute: socket troubles");
-    broadcast_socket(rawsd);
-#ifndef WIN32
-    sethdrinclude(rawsd);
-#endif
     ethsd = NULL;
   }
 
@@ -981,7 +1035,7 @@ void traceroute_hop_cache_clear() {
    and that differences aren't meaningful. (This has the same effect as if we
    were to send probes strictly serially, because then there would be no parent
    hops to potentially conflict, even if in fact they would if traced to
-   completion. */
+   completion.) */
 static Hop *merge_hops(const struct sockaddr_storage *tag, Hop *a, Hop *b) {
   Hop head, *p;
 
@@ -1203,7 +1257,7 @@ static bool decode_reply(const void *ip, unsigned int len, Reply *reply) {
       const struct icmpv6_msg_echo *echo;
 
       if (len < sizeof(*icmpv6) + 4)
-	return false;
+        return false;
       echo = (struct icmpv6_msg_echo *) ((char *) icmpv6 + sizeof(*icmpv6));
       if (ntohs(echo->icmpv6_id) != global_id)
         return false;
