@@ -3,59 +3,91 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+# * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
 # * also a registered trademark of Insecure.Com LLC.  This program is free  *
 # * software; you may redistribute and/or modify it under the terms of the  *
 # * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 with the clarifications and exceptions described  *
-# * below.  This guarantees your right to use, modify, and redistribute     *
-# * this software under certain conditions.  If you wish to embed Nmap      *
-# * technology into proprietary software, we sell alternative licenses      *
-# * (contact sales@insecure.com).  Dozens of software vendors already       *
-# * license Nmap technology such as host discovery, port scanning, OS       *
-# * detection, version detection, and the Nmap Scripting Engine.            *
+# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+# * modify, and redistribute this software under certain conditions.  If    *
+# * you wish to embed Nmap technology into proprietary software, we sell    *
+# * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+# * vendors already license Nmap technology such as host discovery, port    *
+# * scanning, OS detection, version detection, and the Nmap Scripting       *
+# * Engine.                                                                 *
 # *                                                                         *
-# * Note that the GPL places important restrictions on "derived works", yet *
-# * it does not provide a detailed definition of that term.  To avoid       *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
 # * misunderstandings, we interpret that term as broadly as copyright law   *
 # * allows.  For example, we consider an application to constitute a        *
-# * "derivative work" for the purpose of this license if it does any of the *
-# * following:                                                              *
-# * o Integrates source code from Nmap                                      *
-# * o Reads or includes Nmap copyrighted data files, such as                *
-# *   nmap-os-db or nmap-service-probes.                                    *
-# * o Executes Nmap and parses the results (as opposed to typical shell or  *
-# *   execution-menu apps, which simply display raw Nmap output and so are  *
-# *   not derivative works.)                                                *
-# * o Integrates/includes/aggregates Nmap into a proprietary executable     *
-# *   installer, such as those produced by InstallShield.                   *
-# * o Links to a library or executes a program that does any of the above   *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
 # *                                                                         *
-# * The term "Nmap" should be taken to also include any portions or derived *
-# * works of Nmap, as well as other software we distribute under this       *
-# * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
-# * but is meant to clarify our interpretation of derived works with some   *
-# * common examples.  Our interpretation applies only to Nmap--we don't     *
-# * speak for other people's GPL works.                                     *
+# * o Integrates source code from Covered Software.                         *
 # *                                                                         *
-# * If you have any questions about the GPL licensing restrictions on using *
-# * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
-# * we also offer alternative license to integrate Nmap into proprietary    *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
+# *                                                                         *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * permission to link the code of this program with any version of the     *
+# * OpenSSL library which is distributed under a license identical to that  *
+# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+# * linked combinations including the two.                                  *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the special and conditions of the license text as well.       *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, are happy to help.  As mentioned above, we also    *
+# * offer alternative license to integrate Nmap into proprietary            *
 # * applications and appliances.  These contracts have been sold to dozens  *
 # * of software vendors, and generally include a perpetual license as well  *
 # * as providing for priority support and updates.  They also fund the      *
 # * continued development of Nmap.  Please email sales@insecure.com for     *
 # * further information.                                                    *
 # *                                                                         *
-# * As a special exception to the GPL terms, Insecure.Com LLC grants        *
-# * permission to link the code of this program with any version of the     *
-# * OpenSSL library which is distributed under a license identical to that  *
-# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
-# * linked combinations including the two. You must obey the GNU GPL in all *
-# * respects for all of the code used other than OpenSSL.  If you modify    *
-# * this file, you may extend this exception to your version of the file,   *
-# * but you are not obligated to do so.                                     *
-# *                                                                         *
 # * If you received these files with a written license agreement or         *
 # * contract stating terms other than the terms above, then that            *
 # * alternative license agreement takes precedence over these comments.     *
@@ -67,8 +99,8 @@
 # *                                                                         *
 # * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 # * and add new features.  You are highly encouraged to send your changes   *
-# * to nmap-dev@insecure.org for possible incorporation into the main       *
-# * distribution.  By sending these changes to Fyodor or one of the         *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
 # * source code repository, it is understood (unless you specify otherwise) *
 # * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -82,10 +114,9 @@
 # *                                                                         *
 # * This program is distributed in the hope that it will be useful, but     *
 # * WITHOUT ANY WARRANTY; without even the implied warranty of              *
-# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
-# * General Public License v2.0 for more details at                         *
-# * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
-# * included with Nmap.                                                     *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
 # *                                                                         *
 # ***************************************************************************/
 
@@ -106,7 +137,7 @@ from zenmapCore.UmitLogging import log
 class HostSearch(object):
     @staticmethod
     def match_target(host, name):
-        addrs = []
+        name = name.lower()
         mac = host.get_mac()
         ip = host.get_ip()
         ipv6 = host.get_ipv6()
@@ -123,6 +154,7 @@ class HostSearch(object):
         return False
     @staticmethod
     def match_hostname(host, hostname):
+        hostname = hostname.lower()
         hostnames = host.get_hostnames()
         for hn in hostnames:
             if hostname in hn['hostname'].lower():
@@ -146,19 +178,18 @@ class HostSearch(object):
     @staticmethod
     def match_os(host, os):
         os = os.lower()
-        os_str = ""
 
         osmatches = host.get_osmatches()
 
         for osmatch in osmatches:
-            os_str += osmatch['name'].lower()
-            for osclass in osmatch['osmatches']:
-                os_str += osclass['vendor'].lower() + " " +\
+            os_str = osmatch['name'].lower()
+            for osclass in osmatch['osclasses']:
+                os_str += " " + osclass['vendor'].lower() + " " +\
                           osclass['osfamily'].lower() + " " +\
                           osclass['type'].lower()
+            if os in os_str:
+                return True
 
-        if os in os_str:
-            return True
         return False
     @staticmethod
     def match_port(host_ports, port, port_state):
@@ -169,8 +200,8 @@ class HostSearch(object):
         for hp in host_ports:
             if hp['portid'] == port and hp['port_state'] == port_state:
                 return True
-        else:
-            return False
+
+        return False
 
 class SearchResult(object):
     def __init__(self):
@@ -200,7 +231,11 @@ class SearchResult(object):
         """A helper function that calls the matching function for the given
         operator and each of its arguments."""
         for arg in args:
-            if not self.__getattribute__("match_%s" % operator)(arg):
+            positive = True
+            if arg != "" and arg[0] == "!":
+                arg = arg[1:]
+                positive = False
+            if positive != self.__getattribute__("match_%s" % operator)(arg):
                 # No match for this operator
                 return False
         else:
@@ -321,7 +356,6 @@ class SearchResult(object):
         # search just greps through the nmap output, while this function iterates
         # through all parsed OS-related values for every host in every scan!
         hosts = self.parsed_scan.get_hosts()
-        os = os.lower()
         for host in hosts:
             if HostSearch.match_os(host, os):
                 return True
@@ -342,7 +376,7 @@ class SearchResult(object):
         # Make a list of all scanned ports
         services = []
         for scaninfo in self.parsed_scan.get_scaninfo():
-            services = services + scaninfo["services"].split(",")
+            services.append( scaninfo["services"].split(",") )
 
         # These two loops iterate over search ports and over scanned ports. As soon as
         # the search finds a given port among the scanned ports, it breaks from the services
@@ -400,7 +434,6 @@ class SearchResult(object):
         if sversion == "" or sversion == "*":
             return True
 
-        versions = []
         for host in self.parsed_scan.get_hosts():
             if HostSearch.match_service(host, sversion):
                 return True
@@ -410,6 +443,7 @@ class SearchResult(object):
     def match_in_route(self, host):
         if host == "" or host == "*":
             return True
+        host = host.lower()
 
         # Since the parser doesn't parse traceroute output, we need to cheat and look
         # the host up in the Nmap output, in the Traceroute section of the scan.
@@ -425,7 +459,7 @@ class SearchResult(object):
                 traceroutes.append(nmap_out[tr_pos:tr_end_pos])
 
         for tr in traceroutes:
-            if host.lower() in tr.lower():
+            if host in tr.lower():
                 return True
         else:
             return False
@@ -547,27 +581,3 @@ class SearchResultTest(unittest.TestCase):
 
 if __name__ == "__main__":
     unittest.main()
-#    s = SearchDir("/home/adriano/umit/test", ["usr", "xml"])
-#    for result in s.search(\
-#                             keyword="",
-                             #profile="",
-                             #option="",
-                             #started="1121737119",
-                             #finished="1121737192",
-                             #target="10.0.0.100-180",
-                             #mac=":",
-                             #ipv4="10.0.0.150",
-                             #ipv6="",
-                             #uptime=209980,
-                             # lastboot="", MUST BE REMOVED FROM THE UI!
-                             #port=["22", "80"],
-                             #port_open="",
-                             #port_filtered="",
-                             #port_closed="",
-                             #service="",
-                             #osmatch="gentoo",
-                             #product="Apache"\
-#                           ):
-
-#        print "Ports:", result.hosts[-1].ports
-
