@@ -1,29 +1,131 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2005 Insecure.Com LLC.
-#
-# Author: Adriano Monteiro Marques <py.adriano@gmail.com>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+# ***********************IMPORTANT NMAP LICENSE TERMS************************
+# *                                                                         *
+# * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
+# * also a registered trademark of Insecure.Com LLC.  This program is free  *
+# * software; you may redistribute and/or modify it under the terms of the  *
+# * GNU General Public License as published by the Free Software            *
+# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+# * modify, and redistribute this software under certain conditions.  If    *
+# * you wish to embed Nmap technology into proprietary software, we sell    *
+# * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+# * vendors already license Nmap technology such as host discovery, port    *
+# * scanning, OS detection, version detection, and the Nmap Scripting       *
+# * Engine.                                                                 *
+# *                                                                         *
+# * Note that the GPL places important restrictions on "derivative works",  *
+# * yet it does not provide a detailed definition of that term.  To avoid   *
+# * misunderstandings, we interpret that term as broadly as copyright law   *
+# * allows.  For example, we consider an application to constitute a        *
+# * derivative work for the purpose of this license if it does any of the   *
+# * following with any software or content covered by this license          *
+# * ("Covered Software"):                                                   *
+# *                                                                         *
+# * o Integrates source code from Covered Software.                         *
+# *                                                                         *
+# * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+# * or nmap-service-probes.                                                 *
+# *                                                                         *
+# * o Is designed specifically to execute Covered Software and parse the    *
+# * results (as opposed to typical shell or execution-menu apps, which will *
+# * execute anything you tell them to).                                     *
+# *                                                                         *
+# * o Includes Covered Software in a proprietary executable installer.  The *
+# * installers produced by InstallShield are an example of this.  Including *
+# * Nmap with other software in compressed or archival form does not        *
+# * trigger this provision, provided appropriate open source decompression  *
+# * or de-archiving software is widely available for no charge.  For the    *
+# * purposes of this license, an installer is considered to include Covered *
+# * Software even if it actually retrieves a copy of Covered Software from  *
+# * another source during runtime (such as by downloading it from the       *
+# * Internet).                                                              *
+# *                                                                         *
+# * o Links (statically or dynamically) to a library which does any of the  *
+# * above.                                                                  *
+# *                                                                         *
+# * o Executes a helper program, module, or script to do any of the above.  *
+# *                                                                         *
+# * This list is not exclusive, but is meant to clarify our interpretation  *
+# * of derived works with some common examples.  Other people may interpret *
+# * the plain GPL differently, so we consider this a special exception to   *
+# * the GPL that we apply to Covered Software.  Works which meet any of     *
+# * these conditions must conform to all of the terms of this license,      *
+# * particularly including the GPL Section 3 requirements of providing      *
+# * source code and allowing free redistribution of the work as a whole.    *
+# *                                                                         *
+# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * permission to link the code of this program with any version of the     *
+# * OpenSSL library which is distributed under a license identical to that  *
+# * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+# * linked combinations including the two.                                  *
+# *                                                                         *
+# * Any redistribution of Covered Software, including any derived works,    *
+# * must obey and carry forward all of the terms of this license, including *
+# * obeying all GPL rules and restrictions.  For example, source code of    *
+# * the whole work must be provided and free redistribution must be         *
+# * allowed.  All GPL references to "this License", are to be treated as    *
+# * including the special and conditions of the license text as well.       *
+# *                                                                         *
+# * Because this license imposes special exceptions to the GPL, Covered     *
+# * Work may not be combined (even as part of a larger work) with plain GPL *
+# * software.  The terms, conditions, and exceptions of this license must   *
+# * be included as well.  This license is incompatible with some other open *
+# * source licenses as well.  In some cases we can relicense portions of    *
+# * Nmap or grant special permissions to use it in other open source        *
+# * software.  Please contact fyodor@nmap.org with any such requests.       *
+# * Similarly, we don't incorporate incompatible open source software into  *
+# * Covered Software without special permission from the copyright holders. *
+# *                                                                         *
+# * If you have any questions about the licensing restrictions on using     *
+# * Nmap in other works, are happy to help.  As mentioned above, we also    *
+# * offer alternative license to integrate Nmap into proprietary            *
+# * applications and appliances.  These contracts have been sold to dozens  *
+# * of software vendors, and generally include a perpetual license as well  *
+# * as providing for priority support and updates.  They also fund the      *
+# * continued development of Nmap.  Please email sales@insecure.com for     *
+# * further information.                                                    *
+# *                                                                         *
+# * If you received these files with a written license agreement or         *
+# * contract stating terms other than the terms above, then that            *
+# * alternative license agreement takes precedence over these comments.     *
+# *                                                                         *
+# * Source is provided to this software because we believe users have a     *
+# * right to know exactly what a program is going to do before they run it. *
+# * This also allows you to audit the software for security holes (none     *
+# * have been found so far).                                                *
+# *                                                                         *
+# * Source code also allows you to port Nmap to new platforms, fix bugs,    *
+# * and add new features.  You are highly encouraged to send your changes   *
+# * to the dev@nmap.org mailing list for possible incorporation into the    *
+# * main distribution.  By sending these changes to Fyodor or one of the    *
+# * Insecure.Org development mailing lists, or checking them into the Nmap  *
+# * source code repository, it is understood (unless you specify otherwise) *
+# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+# * code.  Nmap will always be available Open Source, but this is important *
+# * because the inability to relicense code has caused devastating problems *
+# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * occasionally relicense the code to third parties as discussed above.    *
+# * If you wish to specify special license conditions of your               *
+# * contributions, just say so when you send them.                          *
+# *                                                                         *
+# * This program is distributed in the hope that it will be useful, but     *
+# * WITHOUT ANY WARRANTY; without even the implied warranty of              *
+# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+# * license file for more details (it's in a COPYING file included with     *
+# * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
+# *                                                                         *
+# ***************************************************************************/
 
 import gobject
 import gtk
 import pango
 import os
 import os.path
+import sys
 import xml.sax
 
 from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog, HIGDialog
@@ -51,31 +153,31 @@ class ScanChooser(HIGVBox):
         "changed": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, ())
     }
 
-    def __init__(self, scan_dict, title):
+    def __init__(self, scans, title):
         self.__gobject_init__()
         self.title = title
-        self.scan_dict = scan_dict
-        
+        self.scan_dict = {}
+
         # Setting HIGVBox
         self.set_border_width(5)
         self.set_spacing(6)
-        
+
         self._create_widgets()
         self._pack_hbox()
         self._attaching_widgets()
         self._set_scrolled()
         self._set_text_view()
         self._set_open_button()
-        
-        for scan in scan_dict:
-            self.list_scan.append([scan])
-        
+
+        for scan in scans:
+            self.add_scan(scan.scan_name or scan.get_nmap_command(), scan)
+
         self.combo_scan.connect('changed', self.show_scan)
         self.combo_scan.connect('changed', lambda x: self.emit('changed'))
-        
+
         self._pack_noexpand_nofill(self.lbl_scan)
         self._pack_expand_fill(self.hbox)
-    
+
     def _create_widgets(self):
         self.lbl_scan = HIGSectionLabel(self.title)
         self.hbox = HIGHBox()
@@ -90,7 +192,7 @@ class ScanChooser(HIGVBox):
 
     def get_buffer(self):
         return self.txt_scan_result.get_buffer()
-    
+
     def show_scan (self, widget):
         nmap_output = self.get_nmap_output()
         if nmap_output is not None:
@@ -107,35 +209,35 @@ class ScanChooser(HIGVBox):
         self.table.attach(self.combo_scan, 0,1,0,1, yoptions=0)
         self.table.attach(self.btn_open_scan, 1,2,0,1, yoptions=0, xoptions=0)
         self.table.attach(self.exp_scan, 0,2,1,2)
-    
+
     def _set_scrolled(self):
         self.scrolled.set_border_width(5)
         self.scrolled.set_size_request(-1, 130)
-        
+
         # Packing scrolled window into expander
         self.exp_scan.add(self.scrolled)
-        
+
         # Packing text view into scrolled window
         self.scrolled.add_with_viewport(self.txt_scan_result)
-        
+
         # Setting scrolled window
         self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-    
+
     def _set_text_view (self):
         self.txg_table = self.txt_scan_result.get_buffer().get_tag_table()
         self.txg_table.add(self.txg_tag)
         self.txg_tag.set_property("family", "Monospace")
-        
+
         self.txt_scan_result.set_wrap_mode(gtk.WRAP_WORD)
         self.txt_scan_result.set_editable(False)
         self.txt_scan_result.get_buffer().connect("changed", self._text_changed_cb)
 
     def _set_open_button (self):
         self.btn_open_scan.connect('clicked', self.open_file)
-    
+
     def open_file (self, widget):
         file_chooser = ResultsFileSingleChooserDialog(_("Select Scan Result"))
-        
+
         response = file_chooser.run()
         file_chosen = file_chooser.get_filename()
         file_chooser.destroy()
@@ -161,7 +263,7 @@ The parsing error that occurred was\n%s") % str(e))
 
             scan_name = os.path.split(file_chosen)[-1]
             self.add_scan(scan_name, parser)
-            
+
             self.combo_scan.set_active(len(self.list_scan) - 1)
 
     def add_scan(self, scan_name, parser):
@@ -170,10 +272,10 @@ The parsing error that occurred was\n%s") % str(e))
         while new_scan_name in self.scan_dict.keys():
             new_scan_name = "%s (%s)" % (scan_name, scan_id)
             scan_id += 1
-                
+
         self.list_scan.append([new_scan_name])
         self.scan_dict[new_scan_name] = parser
-    
+
     def _text_changed_cb (self, widget):
         buff = self.txt_scan_result.get_buffer ()
         buff.apply_tag(self.txg_tag, buff.get_start_iter(), buff.get_end_iter())
@@ -201,11 +303,8 @@ The parsing error that occurred was\n%s") % str(e))
 
 class DiffWindow(gtk.Window):
     def __init__(self, scans):
-        """scans in the format: {"scan_title":parsed_scan}
-        """
         gtk.Window.__init__(self)
         self.set_title(_("Compare Results"))
-        self.scans = scans
         self.ndiff_process = None
         # We allow the user to start a new diff before the old one has finished.
         # We have to keep references to old processes until they finish to avoid
@@ -213,7 +312,16 @@ class DiffWindow(gtk.Window):
         self.old_processes = []
         self.timer_id = None
 
-        self._create_widgets()
+        self.main_vbox = HIGVBox()
+        self.diff_view = DiffView()
+        self.diff_view.set_size_request(-1, 100)
+        self.hbox_buttons = HIGHBox()
+        self.progress = gtk.ProgressBar()
+        self.btn_close = HIGButton(stock=gtk.STOCK_CLOSE)
+        self.hbox_selection = HIGHBox()
+        self.scan_chooser_a = ScanChooser(scans, _(u"A Scan"))
+        self.scan_chooser_b = ScanChooser(scans, _(u"B Scan"))
+
         self._pack_widgets()
         self._connect_widgets()
 
@@ -222,20 +330,9 @@ class DiffWindow(gtk.Window):
         # Initial Size Request
         self.initial_size = self.get_size()
 
-    def _create_widgets(self):
-        self.main_vbox = HIGVBox()
-        self.diff_view = DiffView()
-        self.diff_view.set_size_request(-1, 100)
-        self.hbox_buttons = HIGHBox()
-        self.progress = gtk.ProgressBar()
-        self.btn_close = HIGButton(stock=gtk.STOCK_CLOSE)
-        self.hbox_selection = HIGHBox()
-        self.scan_chooser_a = ScanChooser(self.scans, _(u"A Scan"))
-        self.scan_chooser_b = ScanChooser(self.scans, _(u"B Scan"))
-
     def _pack_widgets(self):
         self.main_vbox.set_border_width(6)
-        
+
         self.hbox_selection.pack_start(self.scan_chooser_a, True, True)
         self.hbox_selection.pack_start(self.scan_chooser_b, True, True)
 
@@ -282,7 +379,7 @@ class DiffWindow(gtk.Window):
             except OSError, e:
                 alert = HIGAlertDialog(
                     message_format = _("Error running ndiff"),
-                    secondary_text = _("There was an error running the ndiff program.\n\n") + str(e))
+                    secondary_text = _("There was an error running the ndiff program.\n\n") + str(e).decode(sys.getdefaultencoding(), "replace"))
                 alert.run()
                 alert.destroy()
             else:
@@ -309,7 +406,7 @@ class DiffWindow(gtk.Window):
                 self.progress.pulse()
                 return True
 
-            if status == 0:
+            if status == 0 or status == 1:
                 # Successful completion.
                 try:
                     diff = self.ndiff_process.get_scan_diff()
@@ -322,7 +419,7 @@ class DiffWindow(gtk.Window):
                 else:
                     self.diff_view.show_diff(diff)
             else:
-                # Unsuccessful completion (non-zero return code).
+                # Unsuccessful completion.
                 error_text = _("The ndiff process terminated with status code %d.") % status
                 stderr = self.ndiff_process.stderr.read()
                 if len(stderr) > 0:
@@ -390,7 +487,7 @@ if __name__ == "__main__":
     parsed2.parse_file("test/xml_test2.xml")
     parsed3.parse_file("test/xml_test3.xml")
     parsed4.parse_file("test/xml_test4.xml")
-    
+
     dw = DiffWindow({"Parsed 1": parsed1,
                      "Parsed 2": parsed2,
                      "Parsed 3": parsed3,
